/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.HeartbeatController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.Log',
    'QDesk.Msg',
    'QDesk.CurrentUser',
    'QDesk.System',

    'QDesk.controller.command.heartbeat.ProcessControlCommands',

    'QDesk.util.FormatUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName'
  ],


  init : function ()
  {
    var me = this;

    var application = me.getApplication();

    application.addListener( QDesk.util.constant.EventName.VIEWPORT_CREATED, me._onViewportCreated, me );
    application.addListener( QDesk.util.constant.EventName.HEARTBEAT, me._onHeartbeat, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  /**
   * Invoked upon receiving a viewportCreated event.
   * @param params event parameters.
   * @private
   */
  _onViewportCreated : function ( params )
  {
    var me = this;

    // start heartbeats after giving the viewport some time to render...
    Ext.defer( me._doHeartbeat, 100, me );
  },


  _onHeartbeat : function ( params )
  {
    var me = this;

    QDesk.Log.debug( 'Heartbeat fired: ' + QDesk.util.FormatUtils.prettyPrint( params ) );

    /*
     * Update the system data.
     */
    QDesk.System.update( params.system );

    /*
     * Update the current user's data (authentication flag + user profile data).
     */
    QDesk.CurrentUser.update( params.sessionToken, params.user, true );

    if ( !Ext.isEmpty( params.notificationMessages ) )
    {
      // TODO: process notification messages
    }

    /*
     * Always process the control commands regardless if there are none as it gives
     * us a chance to unlock the GUI if there are no LOCK commands present.
     */
    Ext.create( 'QDesk.controller.command.heartbeat.ProcessControlCommands', {
      commands : params.controlCommands
    } ).execute();
  },


  /**
   * Performs a heartbeat by invoking the heartbeat action and dispatching a heartbeat event
   * and scheduling repeated execution of this function.
   * @private
   */
  _doHeartbeat : function ()
  {
    var me = this;

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.heartbeatAction,
      callback : function ( opts, success, resp )
      {
        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // we do not need the success flag
            delete json.success;
            me.getApplication().fireEvent( QDesk.util.constant.EventName.HEARTBEAT, json );
          }
          else
          {
            // log error
            QDesk.Log.warn(
                QDesk.Msg.get( 'QDesk.errorExecutingHeartbeatActionMsg',
                    QDesk.util.FormatUtils.prettyPrint( json ) ) );
          }
        }
        else
        {
          QDesk.Log.warn( QDesk.Msg.get( 'QDesk.errorInvokingHeartbeatActionMsg', resp.status, resp.statusText ) );
        }

        /*
         * Heartbeat occurs every 60 seconds.
         */
        Ext.defer( me._doHeartbeat, 60000, me );
      }
    } );
  }
} )
;
