/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Starts all schedulers in the folder whose ID is specified in the configuration parameters.
 * This method also starts all schedulers in all folder's sub-folders.
 */
Ext.define( 'QDesk.controller.command.scheduler.common.StartAllSchedulers', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.model.gui.DialogType',
    'QDesk.model.common.Result',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',

    'QDesk.view.work.scheduler.common.ConnectionsOperationResultsWindow'
  ],

  /*
   * Required ID of the folder containing schedulers to start.
   */
  folderId : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.folderId )
      Ext.Error.raise( 'Required config parameter missing: folderId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    if ( QDesk.ApplicationConfig.get( 'confirm.schedulersStartAll' ) )
    {
      Ext.MessageBox.show( {
        title : QDesk.Msg.get( 'QDesk.util.UxUtils.dialogTitleConfirm' ),

        msg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.common.StartAllSchedulers.confirmMsg',
            Ext.String.htmlEncode( QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId ) ) ),

        buttons : Ext.MessageBox.YESNOCANCEL,
        buttonText : {
          yes : QDesk.Msg.get( 'QDesk.buttonLabelYes' ),
          no : QDesk.Msg.get( 'QDesk.buttonLabelYesIncludeSubfolders' ),
          cancel : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },

        icon : Ext.MessageBox.QUESTION,

        fn : function ( btn )
        {
          if ( btn == 'yes' )
          {
            me._startSchedulers( false );
          }
          else if ( btn == 'no' )
          {
            me._startSchedulers( true );
          }
        }
      } );
    }
    else
    {
      me._startSchedulers( true );
    }
  },


  _startSchedulers : function ( recursive )
  {
    var me = this;

    var folderName = QDesk.controller.command.connection.ConnectionUtils.getFolderName( me.folderId );

    me.mask( QDesk.Msg.get( recursive ?
        'QDesk.controller.command.scheduler.common.StartAllSchedulers.maskRecursiveMsg' :
        'QDesk.controller.command.scheduler.common.StartAllSchedulers.maskNonRecursiveMsg',
        Ext.String.htmlEncode( folderName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.connectionManagementActionStartAll,
      params : {
        folderId : me.folderId,
        recursive : recursive
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            if ( Ext.isEmpty( json.data ) )
            {
              // no schedulers found in the folder
              QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get(
                  recursive ?
                      'QDesk.controller.command.scheduler.common.StartAllSchedulers.warnNoSchedulersFoundRecursiveMsg' :
                      'QDesk.controller.command.scheduler.common.StartAllSchedulers.warnNoSchedulersFoundNonRecursiveMsg',
                  Ext.String.htmlEncode( folderName ) ) );
            }
            else
            {
              var successItems = [];
              var errorItems = [];

              // now we need to check the json.result list to see which schedulers have been started and which have not
              Ext.each( json.data, function ( item )
              {
                if ( item.result == QDesk.model.common.Result.SUCCESS )
                {
                  successItems.push( item );
                }
                else if ( item.result == QDesk.model.common.Result.ERROR )
                {
                  errorItems.push( item );
                }
              } );

              if ( Ext.isEmpty( errorItems ) )
              {
                // no errors - all schedulers have been started
                QDesk.util.UxUtils.showInfo( QDesk.Msg.get( recursive ?
                    'QDesk.controller.command.scheduler.common.StartAllSchedulers.successRecursiveMsg' :
                    'QDesk.controller.command.scheduler.common.StartAllSchedulers.successNonRecursiveMsg',
                    Ext.String.htmlEncode( folderName ) ),
                    function ()
                    {
                      Ext.each( successItems, function ( item )
                      {
                        me.application.fireEvent( QDesk.util.constant.EventName.SCHEDULER_STARTED, {
                          connectionId : item.connectionId
                        } );
                      } );
                    } );
              }
              else
              {
                // some of the schedulers in the folder or its sub-folders were not started
                var window = Ext.widget( 'ConnectionsOperationResultsWindow', {
                  qdesk : {
                    dialogType : QDesk.model.gui.DialogType.WARNING,
                    message : QDesk.Msg.get( recursive ?
                        'QDesk.controller.command.scheduler.common.StartAllSchedulers.warnSomeSchedulersNotStartedRecursiveMsg' :
                        'QDesk.controller.command.scheduler.common.StartAllSchedulers.warnSomeSchedulersNotStartedNonRecursiveMsg',
                        Ext.String.htmlEncode( folderName ) ),
                    results : json.data,
                    callback : function ()
                    {
                      Ext.each( successItems, function ( item )
                      {
                        me.application.fireEvent( QDesk.util.constant.EventName.SCHEDULER_STARTED, {
                          connectionId : item.connectionId
                        } );
                      } );
                    }
                  }
                } );

                window.show();
              }
            }
          }
          else
          {
            // json.success = false
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }

        // reload the folder in the connections tree view
        Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
          folderId : me.folderId
        } ).execute();
      }
    } );
  }
} );
