/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window to view the currently executing job's log.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.ViewExecutingJobLog', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob',
    'QDesk.controller.command.scheduler.quartz.jobs.SaveExecutingJobLog',

    'QDesk.model.license.ProductEdition',
    'QDesk.model.log.LoggingEventPriority',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.ViewLogWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config property containing the executing job.
   */
  executingJob : undefined,

  /**
   * Flag indicating if auto-scroll is enabled.
   */
  autoScrollEnabled : true,

  /**
   * Interval between individual log data fetches in ms.
   */
  fetchInterval : 1000,

  /**
   * Counter of log data fetches.
   */
  fetchCount : 0,

  /**
   * Counter of successive log data fetch errors.
   */
  fetchErrorCount : 0,

  /**
   * Maximum number of successive log data fetch errors before we give up.
   */
  maxFetchErrors : 5,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.executingJob )
      Ext.Error.raise( 'Required config parameter missing: executingJob' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // jobExecutionId is available for Quartz >= 2.1.1
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );
    if ( Ext.isEmpty( jobExecutionId ) )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.jobs.ViewExecutingJobLog.warnViewLogNotSupported' ) );
    }
    else
    {
      var window = Ext.widget( 'ViewLogWindow' );

      window.on( 'afterrender', function ()
      {
        window.maximize();
        window.toFront( false );
      } );

      var viewPanel = window.down( '#viewPanel' );

      /*
       * Set focus on the editor so that the user can immediately use CTRL+A to
       * select all text in the editor. The delay is used to defer setting of the
       * focus to avoid a known issue (see http://forums.ext.net/showthread.php?3315-CLOSED-Set-Focus-to-textfield-in-logon-window).
       */
      QDesk.util.UxUtils.focus( viewPanel, false );

      viewPanel.setHtml(
          '<pre id="logText" style="margin-bottom: 0; margin-top: 0;"></pre><img id="spinner" src="images/icon/loading-16x16.gif" style="display:none"/>' );

      var saveButton = window.down( 'toolbar #saveButton' );
      saveButton.setDisabled( false );
      saveButton.on( 'click', function ()
      {
        me._saveLogData( window );
      } );

      /*
       * Auto-scroll button logic.
       */
      var autoScrollButton = window.down( 'toolbar #autoScrollButton' );
      autoScrollButton.setDisabled( false );
      autoScrollButton.toggle( me.autoScrollEnabled );

      autoScrollButton.on( 'toggle', function ( button, pressed )
      {
        me.autoScrollEnabled = pressed;

        // scroll to the bottom without waiting for the data
        me._maybeScrollToBottom( viewPanel );
      }, me );

      /*
       * Interrupt job button logic.
       */
      var interruptJobButton = window.down( 'toolbar #interruptJobButton' );
      interruptJobButton.setDisabled( false );
      interruptJobButton.on( 'click', function ()
      {
        me._interruptJob( window );
      } );

      window.on( 'show', function ()
      {
        /*
         * We need to wait for the logText element to be available and findable by the getElementById
         * method.
         */
        var waitFn = function ()
        {
          var logTextDom = Ext.dom.Element.get( 'logText' );
          if ( logTextDom )
          {
            me._fetchLogData( window, viewPanel, jobExecutionId, 0 );
          }
          else
          {
            window.requestAnimationFrame( waitFn );
          }
        };

        waitFn();
      } );

      window.show();
    }
  },


  _fetchLogData : function ( window, viewPanel, jobExecutionId, fromIndex )
  {
    var me = this;

    if ( !viewPanel.isVisible() )
      return;

    if ( me.fetchCount === 0 )
    {
      window.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.ViewExecutingJobLog.maskMsg' ) );
    }

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.execHistoryActionExecutingJobLog,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        fromIndex : fromIndex,
        jobExecutionId : jobExecutionId
      },
      scope : me,
      callback : function ( opts, success, resp )
      {
        if ( !viewPanel.isVisible() )
          return;

        if ( me.fetchCount === 0 )
        {
          window.unmask();

          me._showSpinner();
        }

        me.fetchCount++;

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            // reset the error count with each successful fetch
            me.fetchErrorCount = 0;

            var jobStarted = json.data.jobStarted;
            var jobStillExecuting = json.data.jobStillExecuting;
            var loggingInterceptionEnabled = json.data.loggingInterceptionEnabled;
            var loggingEvents = json.data.loggingEvent;

            if ( jobStarted )
            {
              if ( loggingEvents.length > 0 )
              {
                var logTextEl = Ext.dom.Element.get( 'logText' );
                me._appendLoggingEventNodes( logTextEl, loggingEvents );

                // scroll to the bottom of the editor in case the auto-scroll is enabled
                me._maybeScrollToBottom( viewPanel );

                // enable the save button
                viewPanel.up( 'window' ).down( 'toolbar #saveButton' ).setDisabled( false );
              }

              if ( jobStillExecuting )
              {
                // job still running, keep fetching the logging events
//                QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                    ') is still executing. About to fetch new data in ' + me.fetchInterval + ' ms' );

                Ext.defer( me._fetchLogData, me.fetchInterval, me,
                    [ window, viewPanel, jobExecutionId, fromIndex + loggingEvents.length ] );
              }
              else
              {
                // job finished executing - remove the spinner
//                QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                    ') finished executing. Stopping data fetching.' );

                me._hideSpinner();

                // disable the auto-scroll and interrupt buttons
                viewPanel.up( 'window' ).down( 'toolbar #autoScrollButton' ).setDisabled( true );
                viewPanel.up( 'window' ).down( 'toolbar #interruptJobButton' ).setDisabled( true );

                /*
                 * Warn the user about the logging interception not being enabled. We can only determine if the
                 * logging interception was enabled once the job finished executing and there is no intercepted
                 * log data.
                 */
                if ( !loggingInterceptionEnabled )
                {
                  QDesk.util.UxUtils.showWarningDialog(
                      QDesk.Msg.get(
                          'QDesk.controller.command.scheduler.quartz.jobs.ViewExecutingJobLog.warnLogInterceptionNotEnabledMsg' ) );
                }
              }
            }
            else
            {
              // job has not started yet - keep trying
//              QDesk.Log.info( 'Job (job execution ID: ' + jobExecutionId +
//                  ') has not started yet. About to fetch new data in ' + me.fetchInterval + ' ms.' );

              Ext.defer( me._fetchLogData, me.fetchInterval, me, [ window, viewPanel, jobExecutionId, 0 ] );
            }
          }
          else
          {
            me._hideSpinner();

            // application error
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          me.fetchErrorCount++;
          if ( me.fetchErrorCount >= me.maxFetchErrors )
          {
            QDesk.Log.warn( 'Error fetching job execution data (job execution ID: ' + jobExecutionId +
                '). Maximum fetch errors (' + me.maxFetchErrors + ') reached, no further retries will be performed.' );

            me._hideSpinner();

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.jobs.ViewExecutingJobLog.errorFetchDataMsg' ) );
          }
          else
          {
            QDesk.Log.warn( 'Error fetching job execution data (job execution ID: ' + jobExecutionId +
                '). Retrying in ' + me.fetchInterval + ' ms.' );

            Ext.defer( me._fetchLogData, me.fetchInterval, me, [ window, viewPanel, jobExecutionId, fromIndex ] );
          }
        }
      }
    } );
  },


  /**
   * Formats the specified list of logging events and returns list of DOM nodes
   * to be added to the HTML editor's log data content area.
   * @param targetEl a DOM document.
   * @param loggingEvents an array of logging event objects with the following structure:
   * <pre>
   *   {
   *     priority : <log message priority>,
   *     message: <log message>
   *   }
   * </pre>
   * @return the formatted logging events as an HTML string.
   * @private
   */
  _appendLoggingEventNodes : function ( targetEl, loggingEvents )
  {
    var result = [];
    Ext.each( loggingEvents, function ( loggingEvent )
    {
      var priority = loggingEvent.priority;

      var priorityColor;
      if ( priority == QDesk.model.log.LoggingEventPriority.TRACE )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.trace' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.DEBUG )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.debug' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.INFO )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.info' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.WARN )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.warn' )
      }
      else if ( priority == QDesk.model.log.LoggingEventPriority.ERROR )
      {
        priorityColor = QDesk.ApplicationConfig.get( 'viewLog.priorityColors.error' )
      }

      Ext.dom.Helper.append( targetEl, {
        tag : 'span',
        html : Ext.String.htmlEncode( loggingEvent.message ) + '<br/>',
        style : 'color: ' + priorityColor
      } );
    } );

    return result;
  },


  /**
   * Shows the spinner icon.
   * @private
   */
  _showSpinner : function ()
  {
    Ext.dom.Element.get( 'spinner' ).show();
  },


  /**
   * Hides the spinner icon.
   * @private
   */
  _hideSpinner : function ()
  {
    Ext.dom.Element.get( 'spinner' ).hide();
  },


  /**
   * Saves the executing jobs's log data.
   * @param window the view log window.
   * @private
   */
  _saveLogData : function ( window )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.SaveExecutingJobLog', {
      connectionCtx : me.connectionCtx,
      executingJob : me.executingJob,
      maskEl : window.getEl()
    } ).execute();
  },


  /**
   * Scrolls the view panel to the bottom.
   * @param viewPanel the view panel.
   * @private
   */
  _maybeScrollToBottom : function ( viewPanel )
  {
    var me = this;

    if ( me.autoScrollEnabled )
    {
      viewPanel.body.scroll( 'down', Number.MAX_VALUE );
    }
  },


  /**
   * Interrupts the currently executing job.
   * @param window the window with the executing job's log.
   * @private
   */
  _interruptJob : function ( window )
  {
    var me = this;
    Ext.create( 'QDesk.controller.command.scheduler.quartz.jobs.InterruptExecutingJob', {
      connectionCtx : me.connectionCtx,
      executingJob : me.executingJob,
      maskEl : window.getEl()
    } ).execute();
  }
} );
