/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Views the thread info of the specified executing job.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.common.jvm.ThreadState',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.scheduler.common.jvm.ThreadInfoWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required config parameter with the executing job whose thread info should be displayed.
   */
  executingJob : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.executingJob )
      Ext.Error.raise( 'Required config parameter missing: executingJob' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    // jobExecutionId is available for Quartz >= 2.1.1
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );
    if ( Ext.isEmpty( jobExecutionId ) )
    {
      QDesk.util.UxUtils.showWarningDialog(
          QDesk.Msg.get(
              'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.warnViewThreadInfoNotSupported' ) );
    }
    else
    {
      me._viewThreadInfo();
    }
  },


  /**
   * Views the thread info.
   */
  _viewThreadInfo : function ()
  {
    var me = this;

    var jobDetail = me.executingJob.get( 'jobDetail' );

    var jobGroupName = jobDetail.group.name;
    var jobName = jobDetail.name;
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.maskMsg',
        Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobActionGetThreadInfo,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobGroupName : jobGroupName,
        jobName : jobName,
        jobExecutionId : jobExecutionId
      },
      callback : function ( opts, success, resp )
      {
        me.unmask();

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            var window = Ext.widget( 'ThreadInfoWindow', {
              qdesk : {
                connectionCtx : me.connectionCtx,
                jobExecutionId : jobExecutionId
              }
            } );

            window.on( 'refresh', function ()
            {
              me._refresh( window );
            } );

            me._loadGeneralTab( window, json.data );
            me._loadStackTraceTab( window, json.data );
            me._loadLockTab( window, json.data );
            me._loadLockedMonitorsTab( window, json.data );
            me._loadLockedSynchronizersTab( window, json.data );

            window.show();
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  },


  /**
   * Refreshes the contents of the specified ThreadInfoWindow.
   * @param window a ThreadInfoWindow.
   * @private
   */
  _refresh : function ( window )
  {
    var me = this;

    var jobDetail = me.executingJob.get( 'jobDetail' );

    var jobGroupName = jobDetail.group.name;
    var jobName = jobDetail.name;
    var jobExecutionId = me.executingJob.get( 'jobExecutionId' );

    QDesk.util.UxUtils.maskEl( window.getEl(),
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.maskMsg',
            Ext.String.htmlEncode( jobGroupName ), Ext.String.htmlEncode( jobName ) ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.quartz.jobActionGetThreadInfo,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        jobGroupName : jobGroupName,
        jobName : jobName,
        jobExecutionId : jobExecutionId
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskEl( window.getEl() );

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            me._loadGeneralTab( window, json.data );
            me._loadStackTraceTab( window, json.data );
            me._loadLockTab( window, json.data );
            me._loadLockedMonitorsTab( window, json.data );
            me._loadLockedSynchronizersTab( window, json.data );
          }
          else
          {
            QDesk.util.UxUtils.showJsonResponseErrors( json );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsAjaxError( resp );
        }
      }
    } );
  },


  _loadGeneralTab : function ( window, data )
  {
    var form = window.down( 'ThreadInfoGeneralForm' );

    form.down( '#threadId' ).setValue( data.threadId + ' (' +
        QDesk.util.FormatUtils.formatNumberInHex( data.threadId ) + ')' );

    form.down( '#threadGroupName' ).setValue( data.threadGroupName );
    form.down( '#threadName' ).setValue( data.threadName );

    // threadState
    var threadStateField = form.down( '#threadState' );
    switch ( data.threadState )
    {
      case QDesk.model.common.jvm.ThreadState.BLOCKED:
        threadStateField.setValue(
            '<span class="threadStateBlocked" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateBlocked' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      case QDesk.model.common.jvm.ThreadState.NEW:
        threadStateField.setValue(
            '<span class="threadStateNew" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateNew' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      case QDesk.model.common.jvm.ThreadState.RUNNABLE:
        threadStateField.setValue(
            '<span class="threadStateRunnable" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateNew' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      case QDesk.model.common.jvm.ThreadState.TERMINATED:
        threadStateField.setValue(
            '<span class="threadStateTerminated" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateTerminated' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      case QDesk.model.common.jvm.ThreadState.TIMED_WAITING:
        threadStateField.setValue(
            '<span class="threadStateTimedWaiting" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateTimedWaiting' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      case QDesk.model.common.jvm.ThreadState.WAITING:
        threadStateField.setValue(
            '<span class="threadStateWaiting" data-qtip="' +
            Ext.String.htmlEncode( QDesk.Msg.get(
                'QDesk.controller.command.scheduler.quartz.jobs.ViewThreadInfo.tooltipThreadStateWaiting' ) ) +
            '">' + QDesk.util.EnumUtils.getThreadStateName( data.threadState ) + '</span>' );
        break;

      default:
        Ext.Error.raise( 'Unexpected thread state value: ' + data.threadState );
    }

    form.down( '#threadPriority' ).setValue( data.threadPriority );

    form.down( '#threadHashCode' ).setValue( data.threadHashCode + ' (' +
        QDesk.util.FormatUtils.formatNumberInHex( data.threadHashCode ) + ')' );

    form.down( '#daemon' ).setValue( QDesk.util.FormatUtils.formatBoolean( data.daemon ) );
    form.down( '#suspended' ).setValue( QDesk.util.FormatUtils.formatBoolean( data.suspended ) );
    form.down( '#inNative' ).setValue( QDesk.util.FormatUtils.formatBoolean( data.inNative ) );

    form.down( '#blockedCount' ).setValue( data.blockedCount );

    var blockedTimeField = form.down( '#blockedTime' );
    if ( data.blockedTime != null )
    {
      blockedTimeField.setValue( data.blockedTime );
      blockedTimeField.setVisible( true );
    }
    else
    {
      blockedTimeField.setVisible( false );
    }

    form.down( '#waitedCount' ).setValue( data.waitedCount );

    var waitedTimeField = form.down( '#waitedTime' );
    if ( data.waitedTime != null )
    {
      waitedTimeField.setValue( data.waitedTime );
      waitedTimeField.setVisible( true );
    }
    else
    {
      waitedTimeField.setVisible( false );
    }
  },


  _loadStackTraceTab : function ( window, data )
  {
    var form = window.down( 'ThreadInfoStackTraceForm' );

    var formattedStackTrace = QDesk.util.FormatUtils.formatStackTrace( data.stackTrace );
    form.down( '#stackTrace' ).setValue( formattedStackTrace );
  },


  _loadLockTab : function ( window, data )
  {
    var form = window.down( 'ThreadInfoLockForm' );

    var infoMessagePanel = form.down( 'InfoMessagePanel' );
    var displayFields = form.down( '#displayFields' );

    if ( data.lockInfo )
    {
      infoMessagePanel.setVisible( false );
      displayFields.setVisible( true );

      form.down( '#lockName' ).setValue( data.lockName );

      form.down( '#className' ).setValue( data.lockInfo.className );

      form.down( '#identityHashCode' ).setValue( data.lockInfo.identityHashCode +
          ' (' + QDesk.util.FormatUtils.formatNumberInHex( data.lockInfo.identityHashCode ) + ')' );

      // lock owner attributes
      var lockOwnerFieldSet = form.down( '#lockOwnerFieldSet' );
      if ( Ext.isEmpty( data.lockOwnerId ) )
      {
        lockOwnerFieldSet.setVisible( false );
        form.down( '#lockOwnerId' ).setValue( '' );
        form.down( '#lockOwnerName' ).setValue( '' );
      }
      else
      {
        lockOwnerFieldSet.setVisible( true );
        form.down( '#lockOwnerId' ).setValue( data.lockOwnerId +
            ' (' + QDesk.util.FormatUtils.formatNumberInHex( data.lockOwnerId ) + ')' );

        form.down( '#lockOwnerName' ).setValue( data.lockOwnerName );
      }
    }
    else
    {
      infoMessagePanel.setVisible( true );
      displayFields.setVisible( false );
    }
  },


  _loadLockedMonitorsTab : function ( window, data )
  {
    var panel = window.down( 'ThreadInfoLockedMonitorsPanel' );

    var grid = panel.down( 'grid' );

    if ( Ext.isEmpty( data.lockedMonitor ) )
    {
      grid.getStore().loadData( [] );
    }
    else
    {
      grid.getStore().loadData( data.lockedMonitor );
    }
  },


  _loadLockedSynchronizersTab : function ( window, data )
  {
    var panel = window.down( 'ThreadInfoLockedSynchronizersPanel' );

    var grid = panel.down( 'grid' );

    if ( Ext.isEmpty( data.lockedSynchronizer ) )
    {
      grid.getStore().loadData( [] );
    }
    else
    {
      grid.getStore().loadData( data.lockedSynchronizer );
    }
  }
} );
