/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Clones an existing notification rule with the specified ID.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.scheduler.SchedulerType',

    'QDesk.util.CommonUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',
    'QDesk.util.quartz.QuartzExecNotificationRuleUtils',

    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required ID of the notification rule to clone.
   */
  notificationRuleId : undefined,

  /**
   * Optional job group name.
   */
  jobGroupName : undefined,

  /**
   * Optional job name.
   */
  jobName : undefined,

  /**
   * Optional trigger group name.
   */
  triggerGroupName : undefined,

  /**
   * Optional trigger name.
   */
  triggerName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.notificationRuleId )
      Ext.Error.raise( 'Required config parameter missing: notificationRuleId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditExecNotificationRuleWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        mode : QDesk.view.work.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.CLONE
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, true );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    /* Load the form data */
    me.mask(
        QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionGet,
      scope : me,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        notificationRuleId : me.notificationRuleId
      },
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          // override the job group name and job name if specified
          if ( !Ext.isEmpty( me.jobGroupName ) && !Ext.isEmpty( me.jobName ) )
          {
            formPanel.down( '#jobGroupName' ).setValue( me.jobGroupName );
            formPanel.down( '#jobName' ).setValue( me.jobName );
          }

          // override the trigger group name and trigger name if specified
          if ( !Ext.isEmpty( me.triggerGroupName ) && !Ext.isEmpty( me.triggerName ) )
          {
            formPanel.down( '#triggerGroupName' ).setValue( me.triggerGroupName );
            formPanel.down( '#triggerName' ).setValue( me.triggerName );
          }

          formPanel.down( '#name' ).setValue( QDesk.Msg.get( 'QDesk.unnamed' ) );

          // set enabledOnWeekdays checkboxes
          Ext.each( Ext.ComponentQuery.query( '#enabledOnWeekdaysFieldSet checkbox', formPanel ),
              function ( checkbox )
              {
                checkbox.setValue( Ext.Array.contains( json.data.enabledOnWeekday, checkbox.qdesk.value ) );
              } );

          // set enabledFromTime
          if ( json.data.enabledFromTime )
          {
            var enabledFromDate = QDesk.util.FormatUtils.parseDayTime( json.data.enabledFromTime );
            formPanel.down( '#enabledFromTime' ).setValue( enabledFromDate );
          }

          // set enabledToTime fields
          if ( json.data.enabledToTime )
          {
            var enabledToDate = QDesk.util.FormatUtils.parseDayTime( json.data.enabledToTime );
            formPanel.down( '#enabledToTime' ).setValue( enabledToDate );
          }

          me.unmask();
          window.show();
        }
        else
        {
          me.unmask();
          window.close();
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        me.unmask();
        window.close();
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();

    var notificationRule = QDesk.util.quartz.QuartzExecNotificationRuleUtils.createNotificationRule( fieldValues );

    // for cloning we need to reset the notification rule ID
    notificationRule.set( 'id', null );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionAdd,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        notificationRule : QDesk.util.CommonUtils.encode( notificationRule )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule.waitMsg',
          Ext.String.htmlEncode( fieldValues.name ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.notif.exec.CloneExecNotificationRule.successMsg',
                  Ext.String.htmlEncode( fieldValues.name ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.EXEC_NOTIF_RULE_ADDED, {
                  connectionId : me.connectionCtx.getConnectionId(),
                  notificationRule : notificationRule
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
