/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window showing the audit trail entries for the specified notification rule with the specified ID.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.model.audit.AuditTrailEntryCategory',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.audit.AuditTrailWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * Required ID of the notification rule whose audit trail to show.
   */
  notificationRuleId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.notificationRuleId )
      Ext.Error.raise( 'Required config parameter missing: notificationRuleId' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var schedulerInfo = me.connectionCtx.getSchedulerInfo();

    var window = Ext.widget( 'AuditTrailWindow', {
      qdesk : {
        gridId : 'quartz.execNotificationRuleAuditTrailGrid',
        schedulerInfo : schedulerInfo
      }
    } );

    me._bindEventHandlers( window );

    var auditTrailGrid = window.down( '#auditTrailGrid' );

    me._refreshAuditTrailGrid( auditTrailGrid );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      window.maximize();
      window.toFront( false );
    } );

    var auditTrailGrid = window.down( '#auditTrailGrid' );

    // refresh button
    auditTrailGrid.on( 'refresh',
        function ()
        {
          me._refreshAuditTrailGrid( auditTrailGrid );
        } );

    // filters changed
    auditTrailGrid.on( 'changeFilters',
        function ()
        {
          me._refreshAuditTrailGrid( auditTrailGrid );
        } );

    var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

    auditTrailGrid.on( 'loadFailed',
        function ( operation )
        {
          auditTrailGrid.down( '#refreshButton' ).enable();
        } );

    auditTrailGrid.on( 'loadSuccess',
        function ( store, records )
        {
          auditTrailGrid.down( '#refreshButton' ).enable();
          auditTrailGrid.down( '#fetchPeriod' ).enable();
          auditTrailGrid.down( '#maxFetchSizeSelect' ).enable();
          // auditTrailGrid.down( '#fetchRelatedCheckbox' ).enable();

          // enable only if the index is enabled
          if ( auditTrailIndexEnabled )
          {
            auditTrailGrid.down( '#filterQueryText' ).enable();
          }

          // update empty text if there are no records
          var view = auditTrailGrid.getView();
          if ( Ext.isEmpty( records ) )
          {
            view.emptyText =
                QDesk.Msg.get(
                    'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail.emptyTextNoMatchingData' );

            view.refresh();
          }

          // upon the initial load of the store, show the window
          if ( !window.rendered )
          {
            window.show();
          }
        } );
  },


  /**
   * Invoked to refresh the data in the audit trail grid.
   */
  _refreshAuditTrailGrid : function ( auditTrailGrid )
  {
    var me = this;

    var extraStoreParams = {
      connectionId : me.connectionCtx.getConnectionId(),
      category : QDesk.model.audit.AuditTrailEntryCategory.QUARTZ_EXEC_NOTIF_RULE,
      targetObjectId : me.notificationRuleId
    };

    var fetchPeriod = auditTrailGrid.down( '#fetchPeriod' ).getValue();
    Ext.apply( extraStoreParams, {
      fetchPeriod : Ext.encode( fetchPeriod )
    } );

    var maxFetchSize = auditTrailGrid.down( '#maxFetchSizeSelect' ).getValue();
    if ( maxFetchSize !== -1 )  // -1 == unlimited
    {
      Ext.apply( extraStoreParams, {
        maxFetchSize : maxFetchSize
      } );
    }

    var auditTrailIndexEnabled = me.connectionCtx.getJvmAgentInfo().isQuartzAuditTrailIndexEnabled();

    // if audit trail index enabled, we add the filter query text param
    if ( auditTrailIndexEnabled )
    {
      var filterQueryText = auditTrailGrid.down( '#filterQueryText' ).getValue();
      if ( !Ext.isEmpty( filterQueryText ) )
      {
        Ext.apply( extraStoreParams, {
          filterQuery : filterQueryText
        } );
      }
    }

    var store = auditTrailGrid.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.auditTrailActionListForTargetObject;
    store.getProxy().extraParams = extraStoreParams;

    me.mask( QDesk.Msg.get(
        'QDesk.controller.command.scheduler.quartz.notif.exec.ViewExecNotificationRuleAuditTrail.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, successful )
      {
        me.unmask();

        if ( successful )
        {
          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  }
} );
