/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.CommonUtils',
    'QDesk.util.StoreUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.EventName',

    'QDesk.util.quartz.QuartzTriggerUtils',

    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode',
    'QDesk.view.work.scheduler.quartz.triggers.EditTriggerWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /*
   * Required type of the trigger to add (see QDesk.model.scheduler.quartz.QuartzTriggerType).
   */
  triggerType : undefined,

  /**
   * Required job group name.
   */
  jobGroupName : undefined,

  /**
   * Required job name.
   */
  jobName : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( !config.triggerType )
      Ext.Error.raise( 'Required config parameter missing: triggerType' );

    if ( !config.jobGroupName )
      Ext.Error.raise( 'Required config parameter missing: jobGroupName' );

    if ( !config.jobName )
      Ext.Error.raise( 'Required config parameter missing: jobName' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger.maskLoadMsg' ) );

    var window = Ext.widget( 'QuartzEditTriggerWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        mode : QDesk.view.work.scheduler.quartz.triggers.EditTriggerFormMode.ADD,
        triggerType : me.triggerType
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      QDesk.util.UxUtils.focus( nameField, false );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    var formStores = formPanel.getFormStores();

    /* Load the form data */
    QDesk.util.StoreUtils.onAllStoresLoad( formStores,
        function ( successful, firstFailedOperation )
        {
          formPanel.down( '#jobName' ).setValue( me.jobName );
          formPanel.down( '#jobGroupName' ).setValue( me.jobGroupName );

          me.unmask();

          if ( successful )
          {
            window.show();
          }
          else
          {
            QDesk.util.UxUtils.showExtJsOperationError( firstFailedOperation, me.failure );
          }
        } );

    QDesk.util.StoreUtils.loadStores( formStores );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();
    var jobDataMap = formPanel.getJobDataMap();

    var triggerGroupName = fieldValues.group;
    var triggerName = fieldValues.name;

    var trigger = QDesk.util.quartz.QuartzTriggerUtils.createTrigger( fieldValues, jobDataMap );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.triggerActionAdd,
      params : {
        connectionId : me.connectionCtx.getConnectionId(),
        trigger : QDesk.util.CommonUtils.encode( trigger )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger.waitMsg',
          Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.AddTrigger.successMsg',
                  Ext.String.htmlEncode( triggerGroupName ), Ext.String.htmlEncode( triggerName ) ),
              function ()
              {
                me.application.fireEvent( QDesk.util.constant.EventName.QUARTZ.TRIGGER_ADDED, {
                  connectionId : me.connectionCtx.getConnectionId(),
                  trigger : trigger
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action, me.failure );
      }
    } );
  }
} );
