/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Statistics view in the scheduler view panel's Triggers tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.controller.command.scheduler.quartz.SchedulerViewUtils',

    'QDesk.model.scheduler.quartz.stats.QuartzTriggerStatisticsType',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.ExecutionStatistics &&
        me.connectionCtx.getJvmAgentInfo().isReady() )
    {
      var triggersTab = QDesk.controller.command.scheduler.quartz.SchedulerViewUtils.getTriggersTab(
          me.connectionCtx.getConnectionId() );

      var triggersGrid = triggersTab.down( '#triggersGrid' );

      var selectedTriggerWithJobDetails = triggersGrid.getSelectionModel().getSelection();

      var statisticsPanel = triggersTab.down( '#statisticsPanel' );

      var statisticsType = statisticsPanel.getStatisticsType();
      var periodLength = statisticsPanel.getPeriodLength();
      var periodStart = statisticsPanel.getPeriodStart();

      /*
       * Remove chart description text and remove the chart component.
       */
      var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );
      chartDescriptionPanel.setVisible( false );
      chartDescriptionPanel.setMessage( '' );

      statisticsPanel.down( '#chartContainer' ).removeAll();

      // disable all fields in the statistics' panel toolbar
      var toolbar = statisticsPanel.down( 'toolbar' );
      Ext.each( Ext.ComponentQuery.query( 'button, field', toolbar ),
          function ( item )
          {
            item.disable();
          }
      );

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedTriggerWithJobDetails ) )
        {
          // no trigger selected - ignore
        }
        else
        {
          var triggerWithJobDetail = selectedTriggerWithJobDetails[ 0 ];

          if ( statisticsType && periodLength )
          {
            if ( periodStart == null )
            {
              me._periodsNotAvailable( statisticsPanel, triggerWithJobDetail, statisticsType, periodLength );
            }
            else
            {
              me._renderChartAndDescription( statisticsPanel, triggerWithJobDetail, statisticsType, periodLength,
                  periodStart );
            }
          }
        }
      }
      else
      {
        // don't populate - ignore
      }
    }
  },


  _periodsNotAvailable : function ( statisticsPanel, triggerWithJobDetail, statisticsType, periodLength )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar but the period selection controls
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          if ( !Ext.String.endsWith( item.getItemId(), 'PeriodSelect' ) )
            item.enable();
        }
    );

    statisticsPanel._reloadAvailablePeriods();
  },


  _renderChartAndDescription : function ( statisticsPanel, triggerWithJobDetail, statisticsType, periodLength,
      periodStart )
  {
    var me = this;

    var toolbar = statisticsPanel.down( 'toolbar' );

    // enable the refresh button + all fields in the statistics' panel toolbar
    Ext.each( Ext.ComponentQuery.query( '#refreshButton, field', toolbar ),
        function ( item )
        {
          item.enable();
        }
    );

    var chart;

    switch ( statisticsType )
    {
      case QDesk.model.scheduler.quartz.stats.QuartzTriggerStatisticsType.TRIGGER_EXEC_COUNT:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.ExecCountChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );
        break;

      case QDesk.model.scheduler.quartz.stats.QuartzTriggerStatisticsType.TRIGGER_EXEC_TIME:
        chart = Ext.create( 'QDesk.view.work.scheduler.quartz.stats.ExecTimeChart', {
          qdesk : {
            periodLength : periodLength,
            periodStart : periodStart
          }
        } );
        break;

      default:
        Ext.Error.raise( 'Unexpected trigger statistics type: ' + statisticsType );
    }

    var store = chart.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.triggerStatisticsActionChartData;

    var trigger = triggerWithJobDetail.get( 'trigger' );

    store.getProxy().extraParams = {
      connectionId : me.connectionCtx.getConnectionId(),
      statisticsType : statisticsType,
      periodLength : periodLength,
      periodStart : periodStart,
      triggerGroupName : trigger.group.name,
      triggerName : trigger.name
    };

    me.mask( QDesk.Msg.get(
        'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView.maskLoadMsg' ) );

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          var chartDescriptionPanel = statisticsPanel.down( '#chartDescription' );

          chartDescriptionPanel.setMessage(
              QDesk.Msg.get( 'QDesk.model.scheduler.quartz.stats.QuartzTriggerStatisticsType.description.' +
                  statisticsType ) );

          chartDescriptionPanel.setVisible( true );

          statisticsPanel.down( '#chartContainer' ).add( chart );

          // enable remaining buttons (eg. save chart) that have been disabled
          Ext.each( Ext.ComponentQuery.query( 'button', toolbar ),
              function ( item )
              {
                item.enable();
              }
          );

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get(
                  'QDesk.controller.command.scheduler.quartz.triggers.RefreshStatisticsView.errorMsg' ),
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      }
    } );
  }
} );
