/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a window with a grid populated with the next fire times of the specified trigger.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.ActionUrl',

    'QDesk.view.work.scheduler.quartz.ViewNextFireTimesWindow'
  ],

  /**
   * Required {@link QDesk.ConnectionContext} instance.
   */
  connectionCtx : undefined,

  /*
   * Trigger group name.
   */
  triggerGroupName : undefined,

  /*
   * Trigger name.
   */
  triggerName : undefined,

  /*
   * Trigger instance (QuartzTrigger model).
   */
  trigger : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: connectionCtx' );

    if ( config.trigger )
    {
      Ext.apply( config, {
        triggerGroupName : config.trigger.get( 'group' ).name,
        triggerName : config.trigger.get( 'name' )
      } );
    }
    else
    {
      // trigger instance not specified...
      if ( !config.triggerGroupName )
        Ext.Error.raise( 'Required config parameter missing: triggerGroupName' );

      if ( !config.triggerName )
        Ext.Error.raise( 'Required config parameter missing: triggerName' );
    }

    me.callParent( [ config ] );
  },


  execute : function ()
  {
    var me = this;

    var fullTriggerName = null;
    if ( me.triggerGroupName && me.triggerName )
    {
      fullTriggerName = me.triggerGroupName + ' / ' + me.triggerName;
    }

    var window = Ext.widget( 'QuartzViewNextFireTimesWindow', {
      qdesk : {
        connectionCtx : me.connectionCtx,
        triggerName : fullTriggerName
      }
    } );

    me._bindEventHandlers( window );

    var store = window.down( '#quartzViewNextFireTimesGrid' ).getStore();

    if ( me.trigger )
    {
      // used for new, unsaved triggers that have not been registered with the scheduler yet
      store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.triggerActionListNextFireTimes2;
    }
    else
    {
      store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.triggerActionListNextFireTimes;
    }

    me.mask( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.triggers.ViewNextFireTimes.maskLoadMsg' ) );

    // initial load of data
    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        me.unmask();

        if ( success )
        {
          window.show();

          // invoke the success callback handler
          if ( me.success && Ext.isFunction( me.success ) )
          {
            me.success.apply( me );
          }
        }
        else
        {
          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  },


  _bindEventHandlers : function ( window )
  {
    var me = this;

    window.on( 'afterrender', function ()
    {
      //window.maximize();
      window.toFront( false );
    } );

    var grid = window.down( '#quartzViewNextFireTimesGrid' );

    var store = grid.getStore();

    // in before load we add all required parameters
    store.on( 'beforeload', function ( store, operation )
    {
      grid.down( '#refreshButton' ).setDisabled( true );
      grid.down( '#maxResultsSelect' ).setDisabled( true );

      var params = operation.getParams() || {};

      var maxResults = grid.down( '#maxResultsSelect' ).getValue();

      Ext.apply( params, {
        connectionId : me.connectionCtx.getConnectionId(),
        maxResults : maxResults ? maxResults : null
      } );

      if ( me.trigger )
      {
        // used for new, unsaved triggers that have not been registered with the scheduler yet
        Ext.apply( params, {
          trigger : QDesk.util.CommonUtils.encode( me.trigger )
        } );
      }
      else
      {
        Ext.apply( params, {
          triggerGroupName : me.triggerGroupName,
          triggerName : me.triggerName
        } );
      }

      operation.setParams( params );
    } );

    store.on( 'load',
        function ( store, operation )
        {
          grid.down( '#refreshButton' ).setDisabled( false );
          grid.down( '#maxResultsSelect' ).setDisabled( false );
        } );

    grid.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me._refreshGrid( window );
        } );

    grid.down( '#maxResultsSelect' ).on( 'change',
        function ( combo, newValue, oldValue )
        {
          me._refreshGrid( window );
        } );
  },


  /**
   * Refreshes the contents of the message grid.
   * @param window the messages window.
   * @private
   */
  _refreshGrid : function ( window )
  {
    var me = this;

    var grid = window.down( '#quartzViewNextFireTimesGrid' );

    var store = grid.getStore();
    //store.getProxy().url = QDesk.util.constant.ActionUrl.quartz.triggerActionListNextFireTimes;

    store.loadPage( 1, {
      callback : function ( records, operation, success )
      {
        if ( !success )
        {
          store.removeAll( false );

          QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
        }
      }
    } );
  }
} );
