/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Controller that manages the accordion based NavigationPanel. Currently this controller
 * performs the following actions:
 * <ul>
 *   <li>Upon the selection of a child panel of the navigation panel this controller
 *   adds the {@code selected} class to the selected panel's header. Effectively this
 *   means that the title of the selected child panel will be rendered in bold.
 *   </li>
 * </ul>
 */
Ext.define( 'QDesk.controller.navigation.NavigationController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.util.constant.EventName'
  ],


  init : function ()
  {
    var me = this;

    me.control( {
      // key is a component query, value is a listener configuration
      '#navigationPanel > panel' : {
        expand : me._onExpandChildPanel,
        collapse : me._onCollapseChildPanel
      },
      '#navigationPanel' : {
        afterrender : me._onAfterRender
      }
    } );

    var application = me.getApplication();
    application.on( QDesk.util.constant.EventName.WORK_PANEL_ACTIVATED, me._onWorkPanelActivated, me );
    //application.on( QDesk.util.constant.EventName.WORK_PANEL_DEACTIVATED, me._onWorkPanelDeactivated, me );

    me.callParent( arguments );
  },


  _onWorkPanelActivated : function ()
  {
    var me = this;

    // expand the ConnectionsTreePanel panel, it is the default panel to be opened when the WorkPanel gets activated

    var navigationPanel = Ext.ComponentQuery.query( '#navigationPanel' )[ 0 ];

    var connectionsTreePanel = navigationPanel.down( '#connectionsTreePanel' );
    if ( connectionsTreePanel )
    {
      connectionsTreePanel.expand();
    }
  },


  /**
   * Invoked when the navigationPanel's (accordion) child panel is expanded.
   * This method adds the 'selected' CSS class to the title element of the
   * expanded accordion panel.
   * @param childPanel a navigationPanel's child panel.
   */
  _onExpandChildPanel : function ( childPanel )
  {
    var titleHtmlEl = Ext.query( "span.x-panel-header-text", childPanel.getEl().dom )[ 0 ];
    if ( titleHtmlEl )
    {
      new Ext.Element( titleHtmlEl ).addCls( 'selected' );
    }
  },


  /**
   * Invoked when the navigationPanel's (accordion) child panel is collapsed.
   * This method removes the 'selected' CSS class from the title element of the
   * collapsed accordion panel.
   * @param childPanel a navigationPanel's child panel.
   */
  _onCollapseChildPanel : function ( childPanel )
  {
    var titleHtmlEl = Ext.query( "span.x-panel-header-text", childPanel.getEl().dom )[ 0 ];
    if ( titleHtmlEl )
    {
      new Ext.Element( titleHtmlEl ).removeCls( 'selected' );
    }
  },


  /**
   * Invoked when the navigationPanel component renders. This method
   * adds the 'selected' CSS class to the initially selected accordion panel.
   * @param navigPanel a navigationPanel's child panel.
   */
  _onAfterRender : function ( navigPanel )
  {
    var me = this;
    navigPanel.items.each( function ( childPanel, index, childPanels )
    {
      if ( !childPanel.collapsed )
      {
        me._onExpandChildPanel( childPanel );
      }
    } );
  }
} );