/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.scheduler.SchedulerViewController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.controller.command.connection.ConnectionUtils',

    'QDesk.model.scheduler.SchedulerType',

    'QDesk.util.constant.EventName',

    'QDesk.view.work.scheduler.quartz.SchedulerView'
  ],


  init : function ()
  {
    var me = this;

    var application = me.getApplication();

    application.addListener( QDesk.util.constant.EventName.SCHEDULER_STARTED, me._onSchedulerStarted, me );
    application.addListener( QDesk.util.constant.EventName.SCHEDULER_PAUSED, me._onSchedulerPaused, me );
    application.addListener( QDesk.util.constant.EventName.SCHEDULER_STOPPED, me._onSchedulerStopped, me );

    application.addListener( QDesk.util.constant.EventName.CONNECTION_UPDATED, me._onSchedulerConnectionUpdated, me );
    application.addListener( QDesk.util.constant.EventName.CONNECTION_MOVED, me._onSchedulerConnectionMoved, me );
    application.addListener( QDesk.util.constant.EventName.CONNECTION_DELETED, me._onSchedulerConnectionDeleted, me );

    application.addListener( QDesk.util.constant.EventName.CONNECTION_FLD_MOVED, me._onConnectionFolderMoved, me );
    application.addListener( QDesk.util.constant.EventName.CONNECTION_FLD_DELETED, me._onConnectionFolderDeleted, me );

    application.addListener( QDesk.util.constant.EventName.WORK_PANEL_ACTIVATED, me._onWorkPanelActivated, me );
    application.addListener( QDesk.util.constant.EventName.WORK_PANEL_DEACTIVATED, me._onWorkPanelDeactivated, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;
    me.callParent( arguments );
  },


  /**
   * Invoked when a scheduler has been started. This method checks if there is a tab for the
   * given scheduler and if so, it refreshes its sub-panels by firing appropriate events.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  _onSchedulerStarted : function ( eventData )
  {
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      var connectionCtx = schedulerView.getConnectionCtx();
      var schedulerInfo = connectionCtx.getSchedulerInfo();

      switch ( schedulerInfo.getSchedulerType() )
      {
        case QDesk.model.scheduler.SchedulerType.QUARTZ:
          Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView', {
            connectionCtx : connectionCtx
          } ).execute();
          break;

        default:
          Ext.Error.raise( 'Unexpected scheduler type: ' + schedulerInfo.getSchedulerType() );
      }
    }
  },


  /**
   * Invoked when a scheduler has been paused. This method checks if there is a tab for the
   * given scheduler and if so, it refreshes its sub-panels by firing appropriate events.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  _onSchedulerPaused : function ( eventData )
  {
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      var connectionCtx = schedulerView.getConnectionCtx();
      var schedulerInfo = connectionCtx.getSchedulerInfo();

      switch ( schedulerInfo.getSchedulerType() )
      {
        case QDesk.model.scheduler.SchedulerType.QUARTZ:
          Ext.create( 'QDesk.controller.command.scheduler.quartz.RefreshSchedulerView', {
            connectionCtx : schedulerView.getConnectionCtx()
          } ).execute();
          break;

        default:
          Ext.Error.raise( 'Unexpected scheduler type: ' + schedulerInfo.getSchedulerType() );
      }
    }
  },


  /**
   * Invoked when a scheduler has been stopped. This method checks if there is a tab for the
   * given scheduler and if so, it destroys the tab panel.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  _onSchedulerStopped : function ( eventData )
  {
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      schedulerView.close();
    }
  },


  /**
   * Invoked when a connection has been updated. This method checks if there is an open view for the
   * given connection and if so, it closes the view.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  _onSchedulerConnectionUpdated : function ( eventData )
  {
    // close the scheduler view if it is open
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      schedulerView.close();
    }
  },


  /**
   * Invoked when a connection has been moved. This method checks if there is an open view for the
   * given connection and if so, it updates the connectionPath value cached in the view.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   *   targetFolderId       : target folder ID
   *   targetFolderPath     : target folder path (in the Connections tree view)
   * </pre>
   */
  _onSchedulerConnectionMoved : function ( eventData )
  {
    // if the scheduler view for the moved connection is opened, update the connection node path stored in the view
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      var connectionCtx = schedulerView.getConnectionCtx();
      var connectionNodePath = connectionCtx.getConnectionNodePath();

      var i = connectionNodePath.lastIndexOf( '/' );
      if ( i > 0 )
      {
        var newConnectionNodePath = eventData.targetFolderPath + connectionNodePath.substring( i );
        connectionCtx.setConnectionNodePath( newConnectionNodePath );
      }
    }
  },


  /**
   * Invoked when a connection has been deleted. This method checks if there is an open view for the
   * given connection and if so, it closes the view.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  _onSchedulerConnectionDeleted : function ( eventData )
  {
    // close the scheduler view if it is open
    var schedulerView = QDesk.controller.command.connection.ConnectionUtils.getSchedulerView( eventData.connectionId );
    if ( schedulerView )
    {
      schedulerView.close();
    }
  },


  /**
   * Invoked when a folder has been moved. This method checks if there are open views for connections
   * that reside in the moved folder and if so, it updates the connectionPath values cached in these views.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   folderId             : folder ID of the moved folder
   *   folderPath           : folder path of the moved folder
   *   targetFolderId       : target folder ID
   *   targetFolderPath     : target folder path
   * </pre>
   */
  _onConnectionFolderMoved : function ( eventData )
  {
    // find all Quartz scheduler views whose connectionPath starts with the moved folder's path
    var quartzSchedulerViews = Ext.ComponentQuery.query( '#mainTabPanel > QuartzSchedulerView' );
    Ext.each( quartzSchedulerViews, function ( schedulerView )
    {
      var connectionCtx = schedulerView.getConnectionCtx();
      var connectionNodePath = connectionCtx.getConnectionNodePath();

      if ( connectionNodePath.startsWith( eventData.folderPath ) )
      {
        var relativeConnectionNodePath = connectionNodePath.substring( eventData.folderPath.length );

        var i = eventData.folderPath.lastIndexOf( '/' );
        if ( i > 0 )
        {
          var relativeFolderPath = eventData.folderPath.substring( i );
          var newConnectionNodePath = eventData.targetFolderPath + relativeFolderPath + relativeConnectionNodePath;
          connectionCtx.setConnectionNodePath( newConnectionNodePath );
        }
      }
    } );
  },


  /**
   * Invoked when a folder has been deleted. This method checks if there are open views for connections
   * that resided in the deleted folder and if so, it closes the views.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   folderId         : folder ID
   * </pre>
   */
  _onConnectionFolderDeleted : function ( eventData )
  {
    // find all Quartz scheduler views whose connectionPath starts with the deleted folder's path
    var quartzSchedulerViews = Ext.ComponentQuery.query( '#mainTabPanel > QuartzSchedulerView' );
    Ext.each( quartzSchedulerViews, function ( schedulerView )
    {
      var connectionCtx = schedulerView.getConnectionCtx();
      var connectionNodePath = connectionCtx.getConnectionNodePath();

      if ( connectionNodePath.startsWith( eventData.folderPath ) )
      {
        schedulerView.close();
      }
    } );
  },


  _onWorkPanelActivated : function ()
  {
    // do nothing
  },


  _onWorkPanelDeactivated : function ()
  {
    // close all scheduler views
    var quartzSchedulerViews = Ext.ComponentQuery.query( '#mainTabPanel > QuartzSchedulerView' );
    Ext.each( quartzSchedulerViews, function ( schedulerView )
    {
      schedulerView.close();
    } );
  }
} );
