/*
 * Access permission checks in menu items.
 */
Ext.define( 'QDesk.overrides.menu.Item', {
  override : 'Ext.menu.Item',

  requires : [
    'QDesk.util.constant.EventName'
  ],

  /*
   * The standard (desired) menu item state expected by the application.
   */
  _standardStateEnabled : false,

  /*
   * The standard (desired) menu item icon cls expected by the application.
   */
  _standardIconCls : null,

  /*
   * Flag indicating if the current user is authorized to access the menu item.
   */
  _authorized : false,

  initComponent : function ()
  {
    var me = this;

    me.callParent();

    // if ( me.getItemId() === 'addFolderButton' )
    //   debugger;

    // save the standard menu item enabled/disabled state
    me._standardStateEnabled = !me.isDisabled();
    me._standardIconCls = me.iconCls ? me.iconCls : null;

    me._authorized = me._isAuthorized( me.qdRequiresAnyPermissions );

    if ( !me._authorized )
      me._applyNotAuthorized();

    /*
     * QDesk.getApplication exists only after the application has been initialized. During the application
     * initialization, before QDesk.getApplication has been created, ExtJS creates MessageBox button(s)
     * and this causes issues with undefined QDesk.getApplication. This is not the case with menu items, but
     * we check anyway just in case.
     */
    if ( QDesk.getApplication )
    {
      QDesk.getApplication().on( QDesk.util.constant.EventName.CURRENT_USER_PERMISSIONS_CHANGED,
          function ( params )
          {
            if ( me.destroyed )
            {
              // component has been destroyed, do nothing
              return;
            }

            var oldEffectivePermissions = params.oldEffectivePermissions;
            var newEffectivePermissions = params.newEffectivePermissions;

            // if ( me.getItemId() === 'addFolderButton' )
            //   debugger;

            var oldAuthorized = me._authorized;
            var newAuthorized = me._isAuthorized( me.qdRequiresAnyPermissions );

            me._authorized = newAuthorized;

            if ( oldAuthorized !== newAuthorized )
            {
              if ( me._authorized )
              {
                me._applyAuthorized();
              }
              else
              {
                me._applyNotAuthorized();
              }
            }
          } );
    }
  },

  disable : function ( silent )
  {
    var me = this;

    // if ( me.getItemId() === 'addFolderButton' )
    //   debugger;

    /*
     * Change the standardStateEnabled flag only if the change is not silent. ExtJS uses
     * silent disable calls to disable child menu items when their parent menu it is disabled.
     */
    if ( !silent )
    {
      // now the standard menu item state is 'disabled'
      me._standardStateEnabled = false;
    }

    if ( me._authorized )
    {
      // user is authorized - invoke the original disable method
      me.callSuper( arguments );
    }
    else
    {
      me._applyNotAuthorized();
    }
  },

  enable : function ( silent )
  {
    var me = this;

    // if ( me.getItemId() === 'addFolderButton' )
    //   debugger;

    /*
     * Change the standardStateEnabled flag only if the change is not silent. ExtJS uses
     * silent disable calls to disable child menu items when their parent menu it is disabled.
     */
    if ( !silent )
    {
      // now the standard menu item state is 'enabled'
      me._standardStateEnabled = true;
    }

    if ( me._authorized )
    {
      // user is authorized  - invoke the original enable method
      me.callSuper( arguments );
    }
    else
    {
      me._applyNotAuthorized();
    }
  },

  /**
   * This method sets the menu item to the desired (saved) state and restores the original icon
   */
  _applyAuthorized : function ()
  {
    var me = this;

    if ( me.destroyed )
    {
      // do nothing
    }
    else
    {
      // restore the standard menu item icon (if any)
      me.setIconCls( me._standardIconCls );

      // restore the standard menu item state (enabled/disabled)
      if ( me._standardStateEnabled )
      {
        me.superclass.enable.call( me );
      }
      else
      {
        me.superclass.disable.call( me );
      }
    }
  },

  /**
   * This method disables the menu item and makes the menu item use the configured "NAUTH" icon.
   */
  _applyNotAuthorized : function ()
  {
    var me = this;

    if ( me.destroyed )
    {
      // do nothing
    }
    else
    {
      // user is not authorized to use the menu item - disable the menu item and set the new icon CLS
      var newIconCls = me.qdNotAuthorizedIconCls ? me.qdNotAuthorizedIconCls : 'NAUTH';
      me.setIconCls( newIconCls );

      // disable the menu item if not disabled already
      if ( !me.isDisabled() )
      {
        me.superclass.disable.call( me );
      }
    }
  },

  _isAuthorized : function ( requiredAnyPermissions )
  {
    if ( requiredAnyPermissions == null )
    {
      // authentication / authorization not required
      return true;
    }
    else
    {
      if ( Ext.isEmpty( requiredAnyPermissions ) )
      {
        // required permissions are empty, the user must be at least authenticated
        return QDesk.CurrentUser.isAuthenticated();
      }
      else
      {
        return QDesk.CurrentUser.hasAnyPermissions( requiredAnyPermissions );
      }
    }
  }
} );
