/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various common utility methods.
 */
Ext.define( 'QDesk.util.CommonUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,


  /**
   * Returns the current datetime (as a Date object) in the specified time zone. Please note that the returned
   * Date's time zone is always set to the local (browser) time zone.
   * @param timeZone a time zone object with the following structure (see com.quartzdesk.domain.model.common.TimeZone):
   *   {
   *     id : "Europe/Prague",
   *     isoAbbr : "CEST",
   *     displayName : "(UTC+01:00) Europe/Prague (CEST)",
   *     offset : 3600
   *   },
   *
   * @return Date the current time in the specified time zone.
   */
  getCurrentDate : function ( timeZone )
  {
    var currentDate = new Date();

    // compute current time in the UTC time zone
    var utcOffset = currentDate.getTimezoneOffset(); // UTC offset in minutes
    currentDate.setMinutes( currentDate.getMinutes() + utcOffset );

    // compute current time in the scheduler connection's configured time zone
    currentDate.setMinutes( currentDate.getMinutes() + timeZone.offset / 60 );

    return currentDate;
  },


  /**
   * Returns the default fetch period for the specified time zone.
   * @param timeZone a time zone object with the following structure (see com.quartzdesk.domain.model.common.TimeZone):
   *   {
   *     id : "Europe/Prague",
   *     isoAbbr : "CEST",
   *     displayName : "(UTC+01:00) Europe/Prague (CEST)",
   *     offset : 3600
   *   }
   * @param periodLengthInDays the period length in days.
   *
   * @return Object the default fetch period.
   */
  getDefaultFetchPeriod : function ( timeZone, periodLengthInDays )
  {
    var now = QDesk.util.CommonUtils.getCurrentDate( timeZone );
    now.setHours( 0, 0, 0, 0 );

    var to = new Date( now.getTime() );
    to.setDate( to.getDate() + 1 );
    to.setHours( 0, 0, 0, 0 );

    var from = new Date( now.getTime() );
    from.setDate( from.getDate() - periodLengthInDays + 1 );

    return {
      from : from,
      to : to
    };
  },


  /**
   * Returns the number of days between the specified from and to dates.
   * @param from a from date.
   * @param to a to date.
   * @return the number of days.
   */
  getDiffInDays : function ( from, to )
  {
    var timeDiff = Math.abs( to.getTime() - from.getTime() );
    return Math.ceil( timeDiff / ( 1000 * 3600 * 24 ) );
  },


  /**
   * Returns -1, 0, or 1 if the specified version is lower, equal, or greater then the other version.
   * @param version a version.
   * @param otherVersion a minimum version.
   */
  versionCompare : function ( version, otherVersion )
  {
    if ( version.major > otherVersion.major )
    {
      return 1;
    }
    else if ( version.major === otherVersion.major )
    {
      if ( version.minor > otherVersion.minor )
      {
        return 1;
      }
      else if ( version.minor === otherVersion.minor )
      {
        if ( version.maintenance > otherVersion.maintenance )
        {
          return 1;
        }
        else if ( version.maintenance === otherVersion.maintenance )
        {
          return 0;
        }
        else
        {
          // version.maintenance < minVersion.maintenance
          return -1;
        }
      }
      else
      {
        // version.minor < minVersion.minor
        return -1;
      }
    }
    else
    {
      // version.major < minVersion.major
      return -1;
    }
  },


  /**
   * Encodes the data wrapped in the specified ExtJS model instance.
   * This method is useful when POSTing extJS model instances as
   * serialized JSON objects.
   * @param model an ExtJS model instance.
   * @return the JSON serialized model instance.
   */
  encode : function ( model )
  {
    var me = this;
    var clone = me._cloneAndDropEmpty( model.getData() );

    /*
     * If the ID property is not a number, then it means it is the ExtJS generated ID
     * and we need to drop it.
     */
    if ( !Ext.isNumber( clone.id ) )
    {
      delete clone.id;
    }

    return Ext.encode( clone );
  },


  createBasicPropertiesTable : function ( data )
  {
    var html = '<table class="rowDetailsTable basicPropertiesTable">' +
        '<thead>' +
        '<tr><th class="propertyName">' +
        QDesk.Msg.get( 'QDesk.util.CommonUtils.basicPropertiesTable.titlePropertyName' ) +
        '</th><th class="propertyValue">' +
        QDesk.Msg.get( 'QDesk.util.CommonUtils.basicPropertiesTable.titlePropertyValue' ) +
        '</th></tr>' +
        '</thead>' +
        '<tbody>';

    Ext.each( data, function ( row )
    {
      html += '<tr>';
      html += '<td class="propertyName">' + row[ 0 ] + '</td>';
      html += '<td class="propertyValue">' + row[ 1 ] + '</td>';
      html += '</tr>';
    } );

    html += '</tbody>' +
        '</table>';

    return html;
  },


  /**
   * Deep clones the specified object and drops
   * all properties with an empty value.
   * @param o an object to clone.
   * @return the clone result.
   */
  _cloneAndDropEmpty : function ( o )
  {
    var me = this;

    if ( !o || 'object' !== typeof o )
    {
      return o;
    }
    if ( 'function' === typeof o.clone )
    {
      return o.clone();
    }
    var c = '[object Array]' === Object.prototype.toString.call( o ) ? [] : {};
    var p, v;
    for ( p in o )
    {
      if ( o.hasOwnProperty( p ) )
      {
        v = o[ p ];
        if ( v && Ext.typeOf( v ) == 'date' )
        {
          c[ p ] = new Date( v.getTime() );
        }
        else if ( v && 'object' === typeof v )
        {
          c[ p ] = me._cloneAndDropEmpty( v );
        }
        else
        {
          // copy only non-empty properties
          if ( !Ext.isEmpty( v ) )
            c[ p ] = v;
        }
      }
    }
    return c;
  }
} );
