/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various Quartz trigger specific utility methods.
 */
Ext.define( 'QDesk.util.quartz.QuartzTriggerUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.model.scheduler.quartz.QuartzTriggerType',

    'QDesk.model.scheduler.quartz.QuartzTrigger',

    'QDesk.util.CommonUtils',
    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils'
  ],

  singleton : true,


  /**
   * Creates and returns HTML code with trigger details displayed by the rowexpander in the QuartzTriggersGrid.
   * @param values a trigger value object (not a QuartzTrigger model instance!).
   * @param ownerGrid the owner grid.
   * @returns {string}
   */
  createRowDetailsHtml : function ( values, ownerGrid )
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var html = '<table class="rowDetailsTable">' +
        '<tbody>';

    /* Start of Description */
    var description = Ext.isEmpty( values.description ) ? QDesk.Msg.get( 'QDesk.notAvailable' ) : values.description;

    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.description' ) + '</td>';
    html += '<td>' + description + '</td>';
    html += '</tr>';
    /* End of Description */

    /* Start of Monitoring URL */
    html += '<tr>';
    html += '<td style="font-weight: bold; width: 100px;">' + QDesk.Msg.get( 'QDesk.monitoringUrl' ) + '</td>';
    html += '<td>';

    var monitoringLinkUrl = values.monitoringUrl;
    var monitoringLinkTooltip;
    if ( licensedFeatures.PollMonitoring )
    {
      monitoringLinkTooltip =
          Ext.String.htmlEncode(
              QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.tooltipMonitoringUrl' ) );

      html += '<a href="' + monitoringLinkUrl +
          '" class="newWindow" target="_blank" data-qtip="' + monitoringLinkTooltip + '">' + monitoringLinkUrl + '</a>';
    }
    else
    {
      monitoringLinkTooltip = Ext.String.htmlEncode(
          QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.tooltipMonitoringUrlNotLicensed' ) );

      html += '<a href="' + monitoringLinkUrl +
          '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
          monitoringLinkTooltip + '">' + monitoringLinkUrl + '</a>';
    }

    html += '</td></tr>';
    /* End of Monitoring URL */

    html += '</tbody>' +
        '</table>';

    html += '<p/>';
    html += QDesk.util.quartz.QuartzTriggerUtils._createRowDetailsTableHtml( values );

    return html;
  },


  /**
   * Creates a clone of the specified QDesk.model.scheduler.quartz.QuartzTrigger instance.
   * @param trigger a QDesk.model.scheduler.quartz.QuartzTrigger instance.
   * @return the clone.
   */
  cloneTrigger : function ( trigger )
  {
    var clone = trigger.copy();  // does not clone the jobDataMap array

    var jobDataMap = trigger.get( 'jobDataMap' );

    if ( jobDataMap )
    {
      // assert Ext.isArray(jobDataMap)
      clone.set( 'jobDataMap', Ext.Array.clone( jobDataMap ) );
    }

    return clone;
  },


  /**
   * Creates a QDesk.model.scheduler.quartz.QuartzTrigger model instance and populates it with
   * the specified EditXXXTriggerForm field values.
   * @param fieldValues a EditXXXTriggerForm field values.
   * @param jobDataMap an array of QDesk.model.scheduler.quartz.QuartzJobDataMapEntry instances.
   * @return the created QDesk.model.scheduler.quartz.QuartzTrigger model instance.
   */
  createTrigger : function ( fieldValues, jobDataMap )
  {
    var me = this;

    var trigger = Ext.create( 'QDesk.model.scheduler.quartz.QuartzTrigger' );

    if ( !Ext.isEmpty( fieldValues.id ) )
      trigger.set( 'id', fieldValues.id );

    if ( !Ext.isEmpty( fieldValues.type ) )
      trigger.set( 'type', fieldValues.type );

    if ( !Ext.isEmpty( fieldValues.jobGroupName ) )
      trigger.set( 'jobGroup', { name : fieldValues.jobGroupName } );

    if ( !Ext.isEmpty( fieldValues.jobName ) )
      trigger.set( 'jobName', fieldValues.jobName );

    /* Start of common trigger attributes */
    if ( !Ext.isEmpty( fieldValues.name ) )
      trigger.set( 'name', fieldValues.name );

    if ( !Ext.isEmpty( fieldValues.group ) )
      trigger.set( 'group', { name : fieldValues.group } );

    if ( !Ext.isEmpty( fieldValues.description ) )
      trigger.set( 'description', fieldValues.description );

    // start time in the scheduler's (JVM Agent's) time zone
    if ( !Ext.isEmpty( fieldValues.startTime ) )
      trigger.set( 'startTime', fieldValues.startTime );

    // start time in the scheduler's (JVM Agent's) time zone
    if ( !Ext.isEmpty( fieldValues.endTime ) )
      trigger.set( 'endTime', fieldValues.endTime );

    if ( !Ext.isEmpty( fieldValues.priority ) )
      trigger.set( 'priority', fieldValues.priority );

    if ( !Ext.isEmpty( fieldValues.misfireInstruction ) )
      trigger.set( 'misfireInstruction', fieldValues.misfireInstruction );

    if ( !Ext.isEmpty( fieldValues.calendarName ) )
      trigger.set( 'calendar', { id : fieldValues.calendarName } );

    if ( !Ext.isEmpty( jobDataMap ) )
      trigger.set( 'jobDataMap', jobDataMap );

    /* End of common trigger attributes */

    /* Start of trigger-type specific attributes */
    if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.SIMPLE )
    {
      trigger.set( 'repeatCount', fieldValues.repeatCountIndefinitely ? -1 : fieldValues.repeatCount );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.CRON )
    {
      if ( !Ext.isEmpty( fieldValues.cronExpression ) )
        trigger.set( 'cronExpression', fieldValues.cronExpression );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );

      if ( !Ext.isEmpty( fieldValues.preserveHourOfDayAcrossDaylightSavings ) )
        trigger.set( 'preserveHourOfDayAcrossDaylightSavings', fieldValues.preserveHourOfDayAcrossDaylightSavings );

      if ( !Ext.isEmpty( fieldValues.skipDayIfHourDoesNotExist ) )
        trigger.set( 'skipDayIfHourDoesNotExist', fieldValues.skipDayIfHourDoesNotExist );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );

      trigger.set( 'repeatCount', fieldValues.repeatCountIndefinitely ? -1 : fieldValues.repeatCount );

      if ( !Ext.isEmpty( fieldValues.startTimeOfDay ) )
        trigger.set( 'startTimeOfDay', QDesk.util.FormatUtils.formatDayTime( fieldValues.startTimeOfDay ) );

      if ( !Ext.isEmpty( fieldValues.endTimeOfDay ) )
        trigger.set( 'endTimeOfDay', QDesk.util.FormatUtils.formatDayTime( fieldValues.endTimeOfDay ) );

      trigger.set( 'daysOfWeek', me._createWeekdaysArray( fieldValues, 'dayOfWeek' ) );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL )
    {
      if ( !Ext.isEmpty( fieldValues.repeatIntervalUnit ) )
        trigger.set( 'repeatIntervalUnit', fieldValues.repeatIntervalUnit );

      if ( !Ext.isEmpty( fieldValues.repeatInterval ) )
        trigger.set( 'repeatInterval', fieldValues.repeatInterval );
    }
    else if ( fieldValues.type == QDesk.model.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY )
    {
      if ( !Ext.isEmpty( fieldValues.intervalType ) )
        trigger.set( 'intervalType', fieldValues.intervalType );

      if ( !Ext.isEmpty( fieldValues.nthDay ) )
        trigger.set( 'nthDay', fieldValues.nthDay );

      if ( !Ext.isEmpty( fieldValues.nextFireCutoffInterval ) )
        trigger.set( 'nextFireCutoffInterval', fieldValues.nextFireCutoffInterval );

      if ( !Ext.isEmpty( fieldValues.fireAt ) )
        trigger.set( 'fireAt', QDesk.util.FormatUtils.formatDayTime( fieldValues.fireAt ) );

      if ( !Ext.isEmpty( fieldValues.timeZone ) )
        trigger.set( 'timeZone', { id : fieldValues.timeZone } );
    }
    else
    {
      Ext.Error.raise( 'Unexpected trigger type: ' + fieldValues.type );
    }

    /* End of trigger-type specific attributes */

    return trigger;
  },


  /**
   * Creates an array of AddTriggerButton's menu items for the specified Quartz scheduler version.
   * @param schedulerVersion the Quartz scheduler version.
   * @returns {Array} the array of menu items.
   */
  createAddTriggerButtonMenuItems : function ( schedulerVersion )
  {
    var addTriggerMenuItems = [
      {
        itemId : 'addSimpleTrigger',
        text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
            QDesk.model.scheduler.quartz.QuartzTriggerType.SIMPLE ),
//            + ' '
//            + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
        qdesk : {
          triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.SIMPLE
        }
      },
      {
        itemId : 'addCronTrigger',
        text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
            QDesk.model.scheduler.quartz.QuartzTriggerType.CRON ),
//            + ' '
//            + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
        qdesk : {
          triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.CRON
        }
      }
    ];

    // schedulerVersion < 2.0.0
    if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, { major : 2, minor : 0, maintenance : 0 } ) == -1 )
    {
      addTriggerMenuItems.push(
          // <= Quartz 1.8.6
          {
            itemId : 'addDateIntervalTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
                QDesk.model.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL
            }
          }
      );

      addTriggerMenuItems.push(
          // <= Quartz 1.8.6
          {
            itemId : 'addNthIncludedDayTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
                QDesk.model.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY
            }
          }
      );
    }
    // schedulerVersion >= 2.0.0
    else if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, { major : 2, minor : 0, maintenance : 0 } ) >=
        0 )
    {
      addTriggerMenuItems.push(
          // >= Quartz 2.0.0
          {
            itemId : 'addCalendarIntervalTrigger',
            text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
                QDesk.model.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL ),
//                + ' '
//                + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
            qdesk : {
              triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL
            }
          }
      );

      // schedulerVersion >= 2.1.0
      if ( QDesk.util.CommonUtils.versionCompare( schedulerVersion, { major : 2, minor : 1, maintenance : 0 } ) >= 0 )
      {
        addTriggerMenuItems.push(
            // >= Quartz 2.1.0
            {
              itemId : 'addDailyTimeIntervalTrigger',
              text : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName(
                  QDesk.model.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL ),
//                  + ' '
//                  + QDesk.Msg.get( 'QDesk.util.quartz.QuartzUtils.menuAddTriggerButtonTriggerTypeSuffix' ),
              qdesk : {
                triggerType : QDesk.model.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL
              }
            }
        );
      }
    }

    return addTriggerMenuItems;
  },


  /**
   * Creates and returns an HTML table for the specified trigger object.
   * This method is invoked from the trigger grids to obtain a formatted
   * HTML code to display trigger properties.
   * @param values a trigger value object (not a Trigger model instance!).
   * @returns {string}
   */
  _createRowDetailsTableHtml : function ( values )
  {
    switch ( values.type )
    {
      case QDesk.model.scheduler.quartz.QuartzTriggerType.SIMPLE:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatInterval' ),
            values.repeatInterval + ' ms'
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatCount' ),
            ( values.repeatCount === -1 ?
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.repeatCountIndefinitely' ) :
                values.repeatCount )
          ]
        ] );

      case QDesk.model.scheduler.quartz.QuartzTriggerType.CRON:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowCronExpression' ),
            values.cronExpression
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowTimeZone' ),
            ( values.timeZone == null ? '' : values.timeZone.displayName )
          ]
        ] );

      case QDesk.model.scheduler.quartz.QuartzTriggerType.CALENDAR_INTERVAL:
        var rows = [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatIntervalUnit' ),
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( values.repeatIntervalUnit )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatInterval' ),
            values.repeatInterval
          ]
        ];
        // Quartz >= 2.1.3
        if ( !Ext.isEmpty( values.timeZone ) )
        {
          rows.push( [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowTimeZone' ),
            values.timeZone.displayName
          ] );
        }
        // Quartz >= 2.1.3
        if ( !Ext.isEmpty( values.preserveHourOfDayAcrossDaylightSavings ) )
        {
          rows.push( [
            QDesk.Msg.get(
                'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowPreserveHourOfDayAcrossDaylightSavings' ),
            QDesk.util.FormatUtils.formatBoolean( values.preserveHourOfDayAcrossDaylightSavings )
          ] );
        }
        // Quartz >= 2.1.3
        if ( !Ext.isEmpty( values.skipDayIfHourDoesNotExist ) )
        {
          rows.push( [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowSkipDayIfHourDoesNotExist' ),
            QDesk.util.FormatUtils.formatBoolean( values.skipDayIfHourDoesNotExist )
          ] );
        }
        return QDesk.util.CommonUtils.createBasicPropertiesTable( rows );

      case QDesk.model.scheduler.quartz.QuartzTriggerType.DAILY_TIME_INTERVAL:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatIntervalUnit' ),
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( values.repeatIntervalUnit )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatInterval' ),
            values.repeatInterval
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatCount' ),
            ( values.repeatCount === -1 ?
                QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.repeatCountIndefinitely' ) :
                values.repeatCount )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowStartTimeOfDay' ),
            ( Ext.isDate( values.startTimeOfDay ) ?
                QDesk.util.FormatUtils.formatTimeMedium( values.startTimeOfDay ) :
                QDesk.util.FormatUtils.formatTimeMedium(
                    QDesk.util.FormatUtils.parseDayTime( values.startTimeOfDay ) ) )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowEndTimeOfDay' ),
            // values.endTimeOfDay is a Date when using the Trigger ExtJS data model,
            // or String when using the TriggerWithJobDetail data model
            ( Ext.isDate( values.endTimeOfDay ) ?
                QDesk.util.FormatUtils.formatTimeMedium( values.endTimeOfDay ) :
                QDesk.util.FormatUtils.formatTimeMedium(
                    QDesk.util.FormatUtils.parseDayTime( values.endTimeOfDay ) ) )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowDaysOfWeek' ),
            QDesk.util.quartz.QuartzTriggerUtils._formatWeekDays( values.daysOfWeek )
          ]
        ] );

      case QDesk.model.scheduler.quartz.QuartzTriggerType.DATE_INTERVAL:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatIntervalUnit' ),
            QDesk.util.quartz.QuartzEnumUtils.getRepeatIntervalUnitName( values.repeatIntervalUnit )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowRepeatInterval' ),
            values.repeatInterval
          ]
        ] );

      case QDesk.model.scheduler.quartz.QuartzTriggerType.NTH_INCLUDED_DAY:
        return QDesk.util.CommonUtils.createBasicPropertiesTable( [
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowIntervalType' ),
            QDesk.util.quartz.QuartzEnumUtils.getIntervalTypeName( values.intervalType )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowNthDay' ),
            values.nthDay
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowFireAt' ),
            // values.fireAt is a Date when using the Trigger ExtJS data model,
            // or String when using the TriggerWithJobDetail data model
            ( Ext.isDate( values.fireAt ) ?
                QDesk.util.FormatUtils.formatTimeMedium( values.fireAt ) :
                QDesk.util.FormatUtils.formatTimeMedium( QDesk.util.FormatUtils.parseDayTime( values.fireAt ) ) )
          ],
          [
            QDesk.Msg.get( 'QDesk.util.quartz.QuartzTriggerUtils.rowDetailsTable.rowTimeZone' ),
            ( values.timeZone == null ? '' : values.timeZone.displayName )
          ]
        ] );

      default:
        break;
    }
  },


  _formatWeekDays : function ( weekdays )
  {
    if ( Ext.isEmpty( weekdays ) )
    {
      return '';
    }
    else
    {
      var result = '';
      Ext.each( weekdays, function ( weekday, index )
      {
        result += QDesk.util.EnumUtils.getWeekdayName( weekday );
        if ( index < weekdays.length - 1 )
          result += '<br/>';
      } );
      return result;
    }
  },


  /**
   * Returns the array of QDesk.model.common.Weekday instances for the specified form field values.
   * @param fieldValues form field values.
   * @param fieldPrefix the form field name prefix used to denote weekdays.
   * @return the array of QDesk.model.common.Weekday instances.
   */
  _createWeekdaysArray : function ( fieldValues, fieldPrefix )
  {
    var enabledOnWeekdays = [];

    if ( fieldValues[ fieldPrefix + 'Monday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.MONDAY );

    if ( fieldValues[ fieldPrefix + 'Tuesday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.TUESDAY );

    if ( fieldValues[ fieldPrefix + 'Wednesday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.WEDNESDAY );

    if ( fieldValues[ fieldPrefix + 'Thursday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.THURSDAY );

    if ( fieldValues[ fieldPrefix + 'Friday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.FRIDAY );

    if ( fieldValues[ fieldPrefix + 'Saturday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.SATURDAY );

    if ( fieldValues[ fieldPrefix + 'Sunday' ] )
      enabledOnWeekdays.push( QDesk.model.common.Weekday.SUNDAY );

    return enabledOnWeekdays;
  }
} );
