/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various user-related utility methods.
 */
Ext.define( 'QDesk.util.security.UserUtils', {
  singleton : true,

  requires : [
    'QDesk.model.security.User',

    'QDesk.util.MD5',
    'QDesk.util.constant.ActionUrl'
  ],


  /**
   * Returns the UsersTab if it is currently opened, null otherwise.
   * @return the UsersTab if it is currently opened, null otherwise.
   */
  getUsersTab : function ()
  {
    var usersTabs = Ext.ComponentQuery.query( '#mainTabPanel > UsersTab' );
    return Ext.isEmpty( usersTabs ) ? null : usersTabs[ 0 ];
  },


  /**
   * Creates a QDesk.model.security.User model instance and populates it with the specified
   * EditUserForm / EditProfileForm field values.
   * @param fieldValues a EditUserForm / EditProfileForm field values.
   * @return the created QDesk.model.security.User model instance.
   */
  createUser : function ( fieldValues )
  {
    var me = this;

    var user = Ext.create( 'QDesk.model.security.User' );

    // set ID only for an updated user that has a numerical ID; newly added users have textual ExtJS IDs
    if ( Ext.isNumeric( fieldValues.id ) )
    {
      user.set( 'id', fieldValues.id );
    }
    else
    {
      user.set( 'id', null );
    }

    if ( !Ext.isEmpty( fieldValues.system ) )
      user.set( 'system', fieldValues.system );

    if ( !Ext.isEmpty( fieldValues.username ) )
      user.set( 'username', fieldValues.username );

    if ( !Ext.isEmpty( fieldValues.password ) )
      user.set( 'password', fieldValues.password );

    if ( !Ext.isEmpty( fieldValues.firstName ) )
      user.set( 'firstName', fieldValues.firstName );

    if ( !Ext.isEmpty( fieldValues.surname ) )
      user.set( 'surname', fieldValues.surname );

    if ( !Ext.isEmpty( fieldValues.email ) )
      user.set( 'email', fieldValues.email );

    if ( !Ext.isEmpty( fieldValues.workPhoneNumber ) )
      user.set( 'workPhoneNumber', fieldValues.workPhoneNumber );

    if ( !Ext.isEmpty( fieldValues.mobilePhoneNumber ) )
      user.set( 'mobilePhoneNumber', fieldValues.mobilePhoneNumber );

    var avatarType = fieldValues.avatarType;
    user.set( 'avatarType', avatarType );

    var avatarUrl = null;
    switch ( avatarType )
    {
      case QDesk.model.security.UserAvatarType.NONE:
        avatarUrl = null;
        break;

        // case QDesk.model.security.UserAvatarType.LOCAL:
        //   break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
        avatarUrl = fieldValues.avatarExternalCustomUrl;
        break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
        avatarUrl = me._getGravatarUrl( user.get( 'email' ) );
        break;

      default:
        Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
    }
    user.set( 'avatarUrl', avatarUrl );

    if ( !Ext.isEmpty( fieldValues.permissionIds ) )
      user.set( 'permissionIds', fieldValues.permissionIds );

    if ( !Ext.isEmpty( fieldValues.groupIds ) )
      user.set( 'groupIds', fieldValues.groupIds );

    return user;
  },


  /**
   * Updates the user avatar preview image in the specified form using the current form values.
   * @param panel the EditUserForm / EditProfileForm panel.
   */
  updateAvatarPreview : function ( panel )
  {
    var me = this;

    var avatarType = panel.down( '#avatarType' ).getValue().avatarType;

    var emailField = panel.down( '#email' );
    var avatarExternalCustomUrlField = panel.down( '#avatarExternalCustomUrl' );

    var avatarUrl = null;

    switch ( avatarType )
    {
      case QDesk.model.security.UserAvatarType.NONE:
        // use the default avatar image
        break;

        // case QDesk.model.security.UserAvatarType.LOCAL:
        //   break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_CUSTOM_URL:
        if ( avatarExternalCustomUrlField.isValid() )
        {
          avatarUrl = avatarExternalCustomUrlField.getValue();
        }
        else
        {
          // use the default avatar image
        }
        break;

      case QDesk.model.security.UserAvatarType.EXTERNAL_GRAVATAR:
        if ( emailField.isValid() )
        {
          avatarUrl = me._getGravatarUrl( emailField.getValue() );
        }
        else
        {
          // use the default avatar image
        }
        break;

      default:
        Ext.Error.raise( 'Unexpected user avatar type value: ' + avatarType );
    }

    var avatarPreviewField = panel.down( '#avatarPreview' );

    if ( avatarUrl == null )
    {
      // no specific avatar URL, use the default avatar URL
      avatarUrl = 'images/icon/avatar-default-35x35.png';  // default avatar image
      avatarPreviewField.setSrc( avatarUrl );
    }
    else
    {
      // do Ajax HEAD request to check the URL returns an image and set it
      Ext.Ajax.request( {
        url : QDesk.util.constant.ActionUrl.imageActionCheck,
        params : {
          url : avatarUrl
        },
        callback : function ( opts, success, resp )
        {
          if ( success )
          {
            var json = Ext.decode( resp.responseText );
            if ( json.success )
            {
              // all is good, the image exists
            }
            else
            {
              // image does not exist, use the error avatar URL
              avatarUrl = 'images/icon/avatar-error-35x35.png';
            }
          }
          else
          {
            // for some reason we could not check if the image exists, use the error avatar URL
            avatarUrl = 'images/icon/avatar-error-35x35.png';
          }

          avatarPreviewField.setSrc( avatarUrl );
        }
      } );
    }
  },


  /**
   * Returns Gravatar URL for the specified email address.
   * @param email an email address.
   * @returns {string} the Gravatar URL.
   * @private
   */
  _getGravatarUrl : function ( email )
  {
    var avatarUrl = 'https://www.gravatar.com/avatar/';
    avatarUrl += QDesk.util.MD5.md5( email.toLowerCase() );
    //avatarUrl += '?s=35&d=404';
    avatarUrl += '?s=35';
    return avatarUrl;
  }
} );
