/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Panel wrapping various security related forms.
 */
Ext.define( 'QDesk.view.security.SecurityPanel', {
  extend : 'Ext.panel.Panel',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.controller.command.About',

    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProduct',

    'QDesk.view.security.ChangePasswordPanel',
    'QDesk.view.security.ForgotPasswordPanel',
    'QDesk.view.security.ForgotUsernamePanel',
    'QDesk.view.security.LoginPanel',

    'QDesk.view.widget.message.WarningMessagePanel'
  ],

  alias : 'widget.SecurityPanel',

  border : false,

  cls : 'securityPanel',
  bodyCls : 'securityPanelBody',

  layout : 'ux.center',

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.license )
      Ext.Error.raise( 'Required config parameter missing: qdesk.license' );

    var licenseInfo = config.qdesk.license;

    // we do not want to cache the product & user info
    delete config.qdesk;

    Ext.apply( config, {
      items : [
        {
          xtype : 'container',
          border : false,
          cls : 'container',
          width : 350,

          layout : {
            type : 'vbox',
            align : 'stretch',
            pack : 'center'
          },

          items : [
            {
              xtype : 'panel',
              border : false,
              cls : 'logoPanel',
              width : 350,
              height : 100,
              layout : 'fit',
              items : [
                {
                  xtype : 'displayfield',
                  cls : 'productNameAndVersion',
                  renderer : function ()
                  {
                    return QDesk.util.FormatUtils.formatProductNameAndVersion(
                        QDesk.util.license.LicensedProduct.productName,
                        QDesk.util.license.LicensedProduct.productVersion );
                  }
                }
              ]
            },


            /* Message panel */
            {
              xtype : 'WarningMessagePanel',
              itemId : 'warningMessagePanel',
              hidden : true
            },


            /* Forms */
            {
              xtype : 'container',
              itemId : 'formContainer',
              cls : 'formContainer',
              layout : 'card',
              items : [
                {
                  xtype : 'LoginPanel'
                },
                {
                  xtype : 'ChangePasswordPanel'
                },
                {
                  xtype : 'ForgotUsernamePanel'
                },
                {
                  xtype : 'ForgotPasswordPanel'
                }
              ],
              flex : 1
            },


            {
              xtype : 'panel',
              border : false,
              itemId : 'bottomLinks',
              cls : 'bottomLinks',
              layout : {
                type : 'hbox',
                align : 'stretch',
                pack : 'center'
              },
              items : [
                {
                  xtype : 'displayfield',
                  value : '&copy;',
                  margin : '0 5 0 0'
                },
                {
                  xtype : 'displayfield',
                  cls : 'link',
                  itemId : 'linkCopyright',
                  value : QDesk.Msg.get( 'QDesk.view.security.SecurityPanel.bottomLinkCopyright' )

                },
                {
                  xtype : 'displayfield',
                  cls : 'divider',
                  value : '|'
                },
                {
                  xtype : 'displayfield',
                  cls : 'link',
                  itemId : 'linkTermsAndConditions',
                  value : QDesk.Msg.get( 'QDesk.view.security.SecurityPanel.bottomLinkTermsAndConditions' )
                },
                {
                  xtype : 'displayfield',
                  cls : 'divider',
                  value : '|'
                },
                {
                  xtype : 'displayfield',
                  cls : 'link',
                  itemId : 'linkAbout',
                  value : QDesk.Msg.get( 'QDesk.view.security.SecurityPanel.bottomLinkAbout' )
                }
              ]
            }
          ]
        }
      ]
    } );

    me.callParent( [ config ] );

    me.updateLicenseWarnings( licenseInfo );

    me._bindEventHandlers();
  },


  activateLoginPanel : function ()
  {
    var me = this;
    var formContainer = me.down( '#formContainer' );

    var loginPanel = formContainer.down( 'LoginPanel' );
    formContainer.setActiveItem( loginPanel );

    var form = loginPanel.down( 'form' );
    form.reset( true );  // true - clear invalid flags from all fields
    form.isValid();      // revalidate the form after it has been reset

    formContainer.setActiveItem( loginPanel );
  },


  activateChangePasswordPanel : function ()
  {
    var me = this;
    var formContainer = me.down( '#formContainer' );

    var changePasswordPanel = formContainer.down( 'ChangePasswordPanel' );
    var form = changePasswordPanel.down( 'form' );
    form.reset( true );  // true - clear invalid flags from all fields
    form.isValid();      // revalidate the form after it has been reset

    formContainer.setActiveItem( changePasswordPanel );
  },


  activateForgotUsernamePanel : function ()
  {
    var me = this;
    var formContainer = me.down( '#formContainer' );

    var forgotUsernamePanel = formContainer.down( 'ForgotUsernamePanel' );
    var form = forgotUsernamePanel.down( 'form' );
    form.reset( true );  // true - clear invalid flags from all fields
    form.isValid();      // revalidate the form after it has been reset

    formContainer.setActiveItem( forgotUsernamePanel );
  },


  activateForgotPasswordPanel : function ()
  {
    var me = this;
    var formContainer = me.down( '#formContainer' );

    var forgotPasswordPanel = formContainer.down( 'ForgotPasswordPanel' );
    var form = forgotPasswordPanel.down( 'form' );
    form.reset( true );  // true - clear invalid flags from all fields
    form.isValid();      // revalidate the form after it has been reset

    formContainer.setActiveItem( forgotPasswordPanel );
  },


  /**
   * Updates license-related warnings in the warnings panel using the specified license info data.
   * The license info data object is the value of the license attribute from the heartbeat action's response.
   * @param licenseInfo a license info object.
   */
  updateLicenseWarnings : function ( licenseInfo )
  {
    var me = this;

    var warningMessagePanel = me.down( '#warningMessagePanel' );
    var thresholdInDays;
    var warningMessage = null;
    if ( licenseInfo.type === QDesk.model.license.LicenseType.TEMPORARY )
    {
      // warning message is shown only if the license expires in less then the configured number of days (10 by default)
      thresholdInDays =
          QDesk.ApplicationConfig.get( 'notifications.license.temporary.expiryThresholdInDays' );

      if ( thresholdInDays >= licenseInfo.expiresInDays )
      {
        warningMessage = licenseInfo.expiryText;
      }
    }
    else if ( licenseInfo.type === QDesk.model.license.LicenseType.PERPETUAL )
    {
      // warning message is shown only if the SW maintenance expires in less then the configured number of days (30 by default)
      thresholdInDays =
          QDesk.ApplicationConfig.get( 'notifications.license.perpetual.swMaintenanceExpiryThresholdInDays' );

      if ( thresholdInDays >= licenseInfo.swMaintenanceExpiresInDays )
      {
        warningMessage = licenseInfo.swMaintenanceExpiryText;
      }
    }

    warningMessagePanel.setMessage( warningMessage );
    warningMessagePanel.setVisible( !Ext.isEmpty( warningMessage ) );
  },


  /**
   * Binds event handlers.
   */
  _bindEventHandlers : function ()
  {
    var me = this;

    /*
     * Add click events to all links in the bottomLinks panel. These are not HTML links and as such they
     * do not fire click events when clicked.
     */
    Ext.each( Ext.ComponentQuery.query( '#bottomLinks > displayfield[cls=link]', me ),
        function ( link )
        {
          link.on( 'afterrender', function ()
          {
            link.getEl().on( 'click', function ()
            {
              link.fireEvent( 'click' );
            } );
          } );
        } );

    me.down( '#linkCopyright' ).on( 'click', function ()
    {
      window.open( QDesk.Msg.get( 'QDesk.link.productHome' ), '_blank' );
    } );

    me.down( '#linkTermsAndConditions' ).on( 'click', function ()
    {
      var url = null;
      if ( QDesk.util.license.LicensedProduct.productEdition === QDesk.model.license.ProductEdition.LITE )
      {
        url = QDesk.Msg.get( 'QDesk.link.productTermsAndConditionsFree' );
      }
      else
      {
        url = QDesk.Msg.get( 'QDesk.link.productTermsAndConditionsCommercial' );
      }
      window.open( url, '_blank' );
    } );

    me.down( '#linkAbout' ).on( 'click', function ()
    {
      Ext.create( 'QDesk.controller.command.About', {} ).execute();
    } );
  }
} );
