/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.audit.AuditTrailGrid', {
  extend : 'QDesk.view.widget.WConfigurableGrid',

  requires : [
    'QDesk.ApplicationConfig',
    'QDesk.Msg',

    'QDesk.model.security.Permission',

    'QDesk.store.audit.AuditTrailEntryStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.audit.AuditTrailUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.widget.WDateTimePeriod',
    'QDesk.view.widget.WSearchField',

    'Ext.grid.plugin.RowExpander'
  ],

  alias : 'widget.AuditTrailGrid',

  border : false,
  scrollable : true,

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.audit.AuditTrailUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],

  qdesk : {
    /**
     * Required gridId config parameter.
     */
    gridId : undefined,

    /**
     * Optional parameter containing {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined,

    /**
     * Optional config parameter indicating if the "Fetch Related" checkbox should be visible and enabled.
     */
    allowFetchRelated : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.gridId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.gridId' );

    var allowFetchRelated = config.qdesk.allowFetchRelated === true;

    var connectionCtx = config.qdesk.connectionCtx;

    var timeZone = QDesk.ApplicationConfig.get( 'timeZone' );  // default time zone value extracted from app config

    if ( connectionCtx )
    {
      var schedulerInfo = connectionCtx.getSchedulerInfo();
      timeZone = schedulerInfo.getSchedulerTimeZone();
    }

    var fetchPeriod = QDesk.util.CommonUtils.getDefaultFetchPeriod(
        timeZone,
        QDesk.ApplicationConfig.get( config.qdesk.gridId + '.fetchPeriodLength' ) );

    var maxFetchSize = QDesk.ApplicationConfig.get( config.qdesk.gridId + '.maxFetchSize' );

    var store = Ext.create( 'QDesk.store.audit.AuditTrailEntryStore' );

    var tbar = [];

    tbar.push(
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'WDateTimePeriod',
          itemId : 'fetchPeriod',
          allowBlank : false,
          disabled : true,
          value : fetchPeriod,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'combo',
          itemId : 'maxFetchSizeSelect',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.fieldLabelMaxFetchSizeSelect' ),
          store : new Ext.data.ArrayStore( {
            fields : [
              'value',
              'text'
            ],
            data : [
              [ 10, '10' ],
              [ 20, '20' ],
              [ 50, '50' ],
              [ 100, '100' ],
              [ 500, '500' ],
              [ 1000, '1000' ]
              // [-1 , QDesk.Msg.get( 'QDesk.unlimitedForCombo' )]  - disabled, can take a long time to load
            ]
          } ),
          displayField : 'text',
          valueField : 'value',
          queryMode : 'local',
          labelAlign : 'right',
          labelWidth : 68,
          width : 168,
          editable : false,
          disabled : true,
          value : maxFetchSize,
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },

        {
          xtype : 'checkboxfield',
          itemId : 'fetchRelatedCheckbox',
          boxLabel : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.boxLabelFetchRelatedCheckbox' ),
          boxTooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.boxTooltipFetchRelatedCheckbox' ),
          labelAlign : 'right',
          allowBlank : true,
          hidden : !allowFetchRelated,
          disabled : true,
          style : {
            paddingLeft : '10px'
          },
          listeners : {
            change : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },

        {
          xtype : 'WSearchField',
          itemId : 'filterQueryText',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.fieldFilterQueryText' ),
          fieldTooltip : QDesk.Msg.get(
              'QDesk.view.work.audit.AuditTrailGrid.fieldTooltipFilterQueryText' ),
          fieldTooltipClosable : true,  // important because the tooltip contains a link
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.audit.AuditTrailGrid.emptyTextFieldFilterQueryText' ),
          paramName : 'filterQuery',
          labelAlign : 'right',
          labelWidth : 50,
          width : 400,
          hidden : true,  // TODO: at the moment audit trail indices are not supported
          disabled : true,
          listeners : {
            search : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextInvalidFilters' ) );
                }
              }
            },
            clear : {
              fn : function ()
              {
                if ( me.filtersValid() )
                {
                  me.fireEvent( 'changeFilters' );
                }
                else
                {
                  me.empty( QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextInvalidFilters' ) );
                }
              }
            }
          }
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSaveGridConfigButton' ),
          disabled : false,
          qdRequiresAnyPermissions : [ QDesk.model.security.Permission.APP_SETTINGS_UPDATE ],
          qdNotAuthorizedIconCls : 'saveGridConfigNAUTH'
        }
    );

    Ext.apply( config, {
      tbar : tbar,
      columns : [
        {
          itemId : 'idColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipIdColumn' ),
          dataIndex : 'generatedId',
          sortable : false,
          groupable : false,
          flex : 0.7
        },

        {
          itemId : 'createdAtColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textCreatedAtColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipCreatedAtColumn' ),
          dataIndex : 'createdAt',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return value == null ? '' : Ext.String.htmlEncode( value.targetFmtValue );
          }
        },

        {
          itemId : 'categoryColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textCategoryColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipCategoryColumn' ),
          dataIndex : 'category',
          sortable : true,
          groupable : false,
          flex : 1.2,
          /* Need custom renderer to set tooltips */
          renderer : function ( value, metaData, record )
          {
            var displayText = QDesk.util.EnumUtils.getAuditTrailEntryCategoryName( value );
            var tooltipText = displayText /*QDesk.util.EnumUtils.getAuditTrailEntryCategoryName( value )*/;
            var iconClass = 'auditTrailEntryCategory-' + value;

            // img src must be present in order to get rid of the image border that is shown in Chrome
            return '<img src="images/s.gif" align="top" class="x-action-col-icon ' + iconClass +
                ' nonClickable" data-qtip="' + tooltipText + '"/>' +
                '<span style="padding-left: 5px;">' + displayText + '</span>';
          }
        },

        {
          itemId : 'typeColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTypeColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTypeColumn' ),
          dataIndex : 'type',
          sortable : true,
          groupable : false,
          flex : 0.7,
          renderer : function ( value, metaData, record )
          {
            return QDesk.util.EnumUtils.getAuditTrailEntryTypeName( value );
          }
        },

        {
          itemId : 'sessionTokenColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textSessionTokenColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSessionTokenColumn' ),
          dataIndex : 'sessionToken',
          sortable : true,
          groupable : false,
          flex : 1.3,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'sessionOriginColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textSessionOriginColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSessionOriginColumn' ),
          dataIndex : 'sessionOrigin',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value )
          {
            return QDesk.util.EnumUtils.getUserSessionOriginName( value );
          }
        },
        {
          itemId : 'sessionLocalHostColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textSessionLocalHostColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSessionLocalHostColumn' ),
          dataIndex : 'sessionLocalHost',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'sessionHttpServerHostColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textSessionHttpServerHostColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSessionHttpServerHostColumn' ),
          dataIndex : 'sessionHttpServerHost',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'sessionHttpClientHostColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textSessionHttpClientHostColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipSessionHttpClientHostColumn' ),
          dataIndex : 'sessionHttpClientHost',
          sortable : true,
          groupable : false,
          flex : 1
        },

        {
          itemId : 'userIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textUserIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipUserIdColumn' ),
          dataIndex : 'userId',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          itemId : 'externalUserIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textExternalUserIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipExternalUserIdColumn' ),
          dataIndex : 'externalUserId',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'usernameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textUsernameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipUsernameColumn' ),
          dataIndex : 'username',
          sortable : true,
          groupable : false,
          flex : 0.7,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },


        {
          itemId : 'targetObjectColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetObjectColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetObjectColumn' ),
          sortable : true,
          groupable : false,
          flex : 1.3,
          renderer : function ( value, metaData, record )
          {
            var targetObjectId = record.get( 'targetObjectId' );
            var targetObjectName = record.get( 'targetObjectName' );

            if ( Ext.isEmpty( targetObjectName ) )
            {
              return '';
            }
            else
            {
              var displayText = Ext.String.htmlEncode( targetObjectName );
              if ( Ext.isEmpty( targetObjectId ) )
              {
                return '<span>' + displayText + '</span>';
              }
              else
              {
                var tooltipText = '[ID = ' + targetObjectId + ']';
                return '<span data-qtip="' + tooltipText + '">' + displayText + '</span>';
              }
            }
          }
        },

        /* QuartzAuditTrailEntry - specific attributes */
        {
          itemId : 'targetSchedulerObjectNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetSchedulerObjectNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetSchedulerObjectNameColumn' ),
          dataIndex : 'targetSchedulerObjectName',
          sortable : true,
          groupable : false,
          flex : 1.5,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'targetSchedulerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetSchedulerNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetSchedulerNameColumn' ),
          dataIndex : 'targetSchedulerName',
          sortable : true,
          groupable : false,
          flex : 1.3,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'targetSchedulerInstanceIdColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetSchedulerInstanceIdColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetSchedulerInstanceIdColumn' ),
          dataIndex : 'targetSchedulerInstanceId',
          sortable : true,
          groupable : false,
          flex : 1.3,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'targetJobGroupColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetJobGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetJobGroupNameColumn' ),
          dataIndex : 'targetJobGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'targetJobColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetJobNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetJobNameColumn' ),
          dataIndex : 'targetJobName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },

        {
          itemId : 'targetTriggerGroupColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetTriggerGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetTriggerGroupNameColumn' ),
          dataIndex : 'targetTriggerGroupName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        },
        {
          itemId : 'targetTriggerColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.textTargetTriggerNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.tooltipTargetTriggerNameColumn' ),
          dataIndex : 'targetTriggerName',
          sortable : true,
          groupable : false,
          flex : 1,
          renderer : function ( value )
          {
            return Ext.String.htmlEncode( value );
          }
        }
      ],

      store : store,

      qdesk : {
        gridId : config.qdesk.gridId,
        connectionCtx : config.qdesk.connectionCtx
      }
    } );

    me.callParent( [ config ] );
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    me._bindEventHandlers();

    // rows have different CSS style that reflect the refire status
    me.getView().getRowClass = function ( record )
    {
      var category = record.get( 'category' );
      var type = record.get( 'type' );

      if ( QDesk.util.audit.AuditTrailUtils.showDetails( category, type ) )
      {
        /*
         * Do not hide the row expander - details should be shown
         */
        return '';
      }
      else
      {
        /*
         * Hide the row expander - no details should be shown
         */
        return 'hide-row-expander';
      }
    };
  },


  /**
   * Overridden function that returns additional config values to be saved
   * with other grid config values.
   */
  getExtraConfig : function ()
  {
    var me = this;

    var extra = {};

    var fetchPeriod = me.down( '#fetchPeriod' ).getValue();
    if ( fetchPeriod != null )
    {
      var fetchPeriodLength = QDesk.util.CommonUtils.getDiffInDays( fetchPeriod.from, fetchPeriod.to );
      extra[ me.qdesk.gridId + '.fetchPeriodLength' ] = fetchPeriodLength;
    }

    var maxFetchSize = me.down( '#maxFetchSizeSelect' ).getValue();
    extra[ me.qdesk.gridId + '.maxFetchSize' ] = maxFetchSize;

    return extra;
  },


  /**
   * Returns true if the filters in the grid's toolbar are valid, false otherwise.
   * @return true if filters are valid, false otherwise.
   */
  filtersValid : function ()
  {
    var me = this;

    var toolbar = me.down( 'toolbar' );
    if ( !toolbar || !toolbar.rendered )
      return true;

    var fetchPeriodPicker = me.down( '#fetchPeriod' );
    var valid = fetchPeriodPicker.isValid();

    // here we can add other filters

    return valid;
  },


  /**
   * Empties the grid and shows the specified emptyMsg (if not null).
   * @param emptyText an empty message to show.
   */
  empty : function ( emptyText )
  {
    var me = this;

    var store = me.getStore();
    store.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
    store.getProxy().extraParams = {};
    store.loadRecords( [] );

    if ( !Ext.isEmpty( emptyText ) )
    {
      var view = me.getView();
      view.emptyText = emptyText;
      view.refresh();
    }
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    // refresh button
    me.down( '#refreshButton' ).on( 'click',
        function ()
        {
          me.fireEvent( 'refresh' );
        } );

    me.on( 'selectionchange',
        function ( selectionModel, selectedRecords )
        {
          if ( selectedRecords.length === 0 )
          {
            me.fireEvent( 'entryUnselected' );
          }
          else
          {
            me.fireEvent( 'entrySelected', selectedRecords[ 0 ] );
          }
        } );

    var connectionCtx = me.qdesk.connectionCtx;

    var filterQueryText = me.down( '#filterQueryText' );

    var filterQueryEnabled = false;
    if ( connectionCtx )
    {
      var schedulerInfo = connectionCtx.getSchedulerInfo();
      var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

      if ( schedulerInfo.getSchedulerType() == QDesk.model.scheduler.SchedulerType.QUARTZ )
      {
        filterQueryEnabled = jvmAgentInfo.isQuartzAuditTrailIndexEnabled();
      }
    }

    filterQueryText.setDisabled( !filterQueryEnabled );
    if ( !filterQueryEnabled )
    {
      filterQueryText.setEmptyText(
          QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.auditTrailIndexDisabledEmptyText' ) );
    }

    me.getStore().on( 'beforeload',
        function ( store, operation )
        {
          me.down( '#refreshButton' ).disable();
          me.down( '#fetchPeriod' ).disable();
          me.down( '#maxFetchSizeSelect' ).disable();
          me.down( '#fetchRelatedCheckbox' ).disable();
          me.down( '#filterQueryText' ).disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          me.getSelectionModel().deselectAll();
        } );


    me.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          var view = me.getView();

          if ( successful )
          {
            me.fireEvent( 'loadSuccess', store, records );

            // invoke the success callback handler
            if ( me.success && Ext.isFunction( me.success ) )
            {
              me.success.apply( me );
            }
          }
          else
          {
            store.removeAll();

            view.emptyText = QDesk.Msg.get( 'QDesk.view.work.audit.AuditTrailGrid.emptyTextErrorLoadingData' );
            view.refresh();

            me.fireEvent( 'loadFailed', operation );

            QDesk.util.UxUtils.showExtJsOperationError( operation, me.failure );
          }
        } );
  }
} );
