/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for a scheduler view tabs. This class provides access to the
 * tab initialization flag and scheduler description object in the scheduler view.
 */
Ext.define( 'QDesk.view.work.scheduler.BaseSchedulerXPanel', {
  extend : 'QDesk.view.widget.panel.XPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.license.LicensedProduct'
  ],

  bodyCls : 'baseSchedulerXPanelBody',

  closable : false,

  scrollable : true,

  qdesk : {
    /*
     * Required config parameter with the {@link QDesk.ConnectionContext} instance.
     */
    connectionCtx : undefined
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionCtx )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionCtx' );

    me.callParent( [ config ] );
  },


  /**
   * Returns the {@link QDesk.ConnectionContext} instance associated with this tab panel.
   * @return the {@link QDesk.ConnectionContext} instance.
   */
  getConnectionCtx : function ()
  {
    return this.qdesk.connectionCtx;
  },


  /**
   * Returns the QuartzDesk JVM Agent availability and compatibility warning message
   * that gets shown in the yellow box in the GUI if the agent is either not available,
   * or its version/edition is not compatible with the GUI.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getJvmAgentWarningMessage : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk JVM Agent availability and compatibility warning tooltip that gets shown in GUI
   * if the agent is either not available, or its version/edition is not compatible with the GUI.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getJvmAgentWarningTooltip : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( jvmAgentInfo.isReady() )
      return null;

    if ( jvmAgentInfo.isDetected() )
    {
      if ( !jvmAgentInfo.isCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ),
            QDesk.util.FormatUtils.formatVersion( QDesk.util.license.LicensedProduct.productVersion ) );
      }
      else if ( !jvmAgentInfo.isCompatibleEdition() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentIncompatibleEditionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            jvmAgentInfo.getEdition(),
            QDesk.util.license.LicensedProduct.productEdition );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk Public API availability and compatibility warning message that gets shown in the
   * yellow box in the GUI if the API is either not available, is not loaded correctly, or its version is not
   * compatible with the installed QuartzDesk JVM Agent.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getPublicApiWarningMessage : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionMsg',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedMsg',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  },


  /**
   * Returns the QuartzDesk Public API availability and compatibility warning tooltip that gets shown in the
   * the GUI if the API is either not detected, is not loaded correctly, or its version is not compatible
   * with the installed QuartzDesk JVM Agent.
   * @param connectionCtx a {@link QDesk.ConnectionContext} instance.
   * @returns the warning message.
   */
  getPublicApiWarningTooltip : function ( connectionCtx )
  {
    var schedulerInfo = connectionCtx.getSchedulerInfo();
    var jvmAgentInfo = connectionCtx.getJvmAgentInfo();

    if ( schedulerInfo.isPublicApiReady() )
      return null;

    if ( schedulerInfo.isPublicApiDetected() )
    {
      if ( !schedulerInfo.isPublicApiLoadedCorrectly() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotLoadedCorrectlyTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
      }
      else if ( !schedulerInfo.isPublicApiCompatibleVersion() )
      {
        return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiIncompatibleVersionTooltip',
            QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ),
            QDesk.util.FormatUtils.formatVersion( schedulerInfo.getPublicApiVersion() ),
            QDesk.util.FormatUtils.formatVersion( jvmAgentInfo.getVersion() ) );
      }
      else
      {
        return null;
      }
    }
    else
    {
      return QDesk.Msg.get( 'QDesk.featureAccessRestricted.publicApiNotDetectedTooltip',
          QDesk.util.EnumUtils.getSchedulerTypeName( schedulerInfo.getSchedulerType() ) );
    }
  }
} );
