/*
 * Copyright (c) 2013-2020 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid', {
  extend : 'Ext.grid.Panel',

  requires : [
    'QDesk.Msg',

    'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore',
    'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore',

    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzSelectTriggerGrid',

  viewConfig : {
    stripeRows : true
  },

  scrollable : true,

  //border : false,

  qdesk : {
    /**
     * Required connection ID.
     */
    connectionId : undefined,


    /**
     * Optional group name of the trigger to preselect in the grid when it is first loaded.
     */
    preselectedTriggerGroupName : undefined,

    /**
     * Optional name of the trigger to preselect in the grid when it is first loaded.
     */
    preselectedTriggerName : undefined
  },

  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '{[QDesk.util.quartz.QuartzTriggerUtils.createRowDetailsHtml( values, this.owner.grid )]}'
      ]
    }
  ],


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var store = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerWithJobDetailStore' );

    store.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : store,

      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.tooltipRefreshButton' ),
          disabled : true,
          qdRequiresAnyPermissions : [],
          qdNotAuthorizedIconCls : 'refreshNAUTH'
        },
        '-',
        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.fieldLabelFilterByTriggerGroupCombo' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get(
              'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true,
          tpl : Ext.create( 'Ext.XTemplate',
              '<ul class="x-list-plain"><tpl for=".">',
              '<li role="option" class="x-boundlist-item">{name:htmlEncode}</li>',
              '</tpl></ul>' )
        }
      ],

      columns : [
        {
          itemId : 'nameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.textNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.tooltipNameColumn' ),
          dataIndex : 'trigger.name',
          sortable : true,
          groupable : false,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.name );
          }
        },
        {
          itemId : 'groupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.textGroupNameColumn' ),
          tooltip : QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.tooltipGroupNameColumn' ),
          dataIndex : 'group.name',
          sortable : true,
          groupable : true,
          flex : 0.5,
          renderer : function ( value, metaData, record )
          {
            var trigger = record.get( 'trigger' );
            return Ext.String.htmlEncode( trigger.group.name );
          }
        }
      ]
    } );

    me.callParent( [ config ] );

    me._bindEventHandlers();
  },


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var triggerGroupStore = Ext.create( 'QDesk.store.scheduler.quartz.QuartzTriggerGroupStore' );

    triggerGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByTriggerGroupCombo' ).setStore( triggerGroupStore );
  },


  /**
   * Changes the scheduler connection of the grid's store and reloads it.
   * @param connectionId a scheduler connection ID.
   */
  changeConnection : function ( connectionId )
  {
    var me = this;
    var store = me.getStore();

    //
    // We cannot change store parameters while the store is loading.
    // If the store is loading, we reinvoke this method later.
    //
    if ( store.isLoading() )
    {
      Ext.defer( me.changeConnection, 200, me, [ connectionId ] );  // try again in 200ms
    }
    else
    {
      store.removeAll();

      if ( connectionId == null )
      {
        me.setDisabled( true );
      }
      else
      {
        me.setDisabled( false );

        store.getProxy().extraParams = {
          connectionId : connectionId
        };

        var filterByTriggerGroupComboStore = me.down( '#filterByTriggerGroupCombo' ).getStore();
        filterByTriggerGroupComboStore.getProxy().extraParams = {
          connectionId : connectionId
        };

        me.refresh();
      }
    }
  },


  /**
   * Refreshes this grid by first reloading the trigger group combo and then reloading this grid.
   */
  refresh : function ()
  {
    var me = this;

    var filterByTriggerGroupCombo = me.down( '#filterByTriggerGroupCombo' );

    // reloads and resets the trigger group filter combo and then reloads the triggers grid
    filterByTriggerGroupCombo.getStore().load( {
      scope : me,
      callback : function ( records, operation, success )
      {
        if ( success )
        {
          // reload the triggers grid
          if ( filterByTriggerGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            me.getStore().loadPage( 1 );
          }
          else
          {
            // fires the change event which reloads the triggers grid
            filterByTriggerGroupCombo.reset( null );
          }
        }
      }
    } );
  },


  _bindEventHandlers : function ()
  {
    var me = this;

    var refreshButton = me.down( '#refreshButton' );

    // refresh button
    refreshButton.on( 'click',
        function ()
        {
          me.refresh();
        } );

    var store = me.getStore();

    var filterByTriggerGroupCombo = me.down( '#filterByTriggerGroupCombo' );

    filterByTriggerGroupCombo.on( 'change',
        function ( combo, newValue )
        {
          store.clearFilter( true );

          if ( newValue != null )
          {
            store.addFilter( [
              Ext.create( 'Ext.util.Filter', {
                property : 'trigger.group.name',
                value : newValue
              } )
            ], false );
          }

          // apply, or clear the filter and load the first page
          store.loadPage( 1 );
        } );

    filterByTriggerGroupCombo.getStore().on( 'load',
        function ( store, records, successful, operation )
        {
          if ( successful )
          {
            // ok
          }
          else
          {
            QDesk.util.UxUtils.showExtJsOperationError( operation );
          }
        } );

    store.on( 'beforeload',
        function ( store, operation )
        {
          refreshButton.disable();
          filterByTriggerGroupCombo.disable();

          /*
           * FIXME: ExtJS 4.2 does not trigger selectionchange when a grid's store is reloaded.
           * Therefore we need to force-fire this event by clearing the selection.
           */
          me.getSelectionModel().deselectAll();
        } );

    var isFirstSuccessfulStoreLoad = true;

    store.on( 'load',
        function ( store, records, successful, operation )
        {
          var view = me.getView();

          refreshButton.enable();

          if ( successful )
          {
            filterByTriggerGroupCombo.enable();

            if ( Ext.isEmpty( records ) )
            {
              view.emptyText =
                  QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.emptyTextNoMatchingData' );

              view.refresh();
            }
            else
            {
              // upon the first store load, preselect the desired trigger if such a trigger is specified
              if ( isFirstSuccessfulStoreLoad &&
                  !Ext.isEmpty( me.qdesk.preselectedTriggerName ) &&
                  !Ext.isEmpty( me.qdesk.preselectedTriggerGroupName ) )
              {
                var storeIdx = me.getStore().findBy(
                    function ( record, id )
                    {
                      var trigger = record.get( 'trigger' );
                      return trigger.name == me.qdesk.preselectedTriggerName &&
                          trigger.group.name == me.qdesk.preselectedTriggerGroupName;
                    } );

                if ( storeIdx > -1 )
                {
                  var record = me.getStore().getAt( storeIdx );

                  /*
                   * It seems the grid needs some time to finish rendering records before the scroll 100% works.
                   */
                  Ext.defer(
                      function ()
                      {
                        me.getSelectionModel().select( [ record ] );

                        // scroll to view the record
                        view.focusRow( record );
                      }, QDesk.util.constant.Const.DELAY_GRID_SELECT );
                }
              }
            }

            isFirstSuccessfulStoreLoad = false;
          }
          else
          {
            store.removeAll();

            view.emptyText =
                QDesk.Msg.get( 'QDesk.view.work.scheduler.quartz.SelectTriggerGrid.emptyTextErrorLoadingData' );

            view.refresh();

            QDesk.util.UxUtils.showErrorDialog(
                QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.SelectTrigger.errorLoadingTriggersMsg' ) );
          }
        } );
  }
} );
